/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef ArrayHeader
#define ArrayHeader

using namespace std;





//grid based properties
extern double **Caux1;
extern double **Caux2;
extern double **Cauxint;
extern double **Plasmodesmatapattern;
extern int **Tc;
extern int **Tmt;
extern int **Tct;
extern double **PinPrePat;
extern double **Pin2PrePat;
extern double **Cpin1M;
extern double **Cpin2M;
extern double **Aux1PrePat;
extern double **Aux1M;
extern double **Pltsgrid;
extern double **Pltsgrid2;
extern double **dummypict;
extern double *BaseMeanCell;
extern double *TempAuxin;
extern double *IncrAuxin;
extern bool *Auxminpassed;
extern bool *OldAuxminpassed;
extern bool *ARRpassed;
extern bool *OldARRpassed;
//cells on grid
extern int **Cell;
//cell based properties
extern int *CellMinI;
extern int *CellMaxI;
extern int *CellMinJ;
extern int *CellMaxJ;
extern int *CellWidth;
extern int *CellHeight;
extern int *CellSizes;
extern double *InvCellSizes;
extern double *MeanCells;
extern double *OldMeanCells;
extern double *BaselineAuxine;
extern double *DiffAuxine;
extern double *TimeOfAuxineIncrease;
//cell based gene expression
extern double *ARF;
extern double *AA;
extern double *BB;
extern double *CC;
extern double *DD;
extern double *Plt;
extern double *Pltexpr;
extern double *Plts;
extern double *Pltsexpr;
extern double *PP;
extern double *Pin;
extern double *Pin2;
extern double *Aux1;
extern double *Yuc3;
extern double *ARR1;
extern double *ARR1RGA;
extern double *ARR12;

extern double *Gh317;
extern double *Gh33;
extern double *divTF;
extern double *elongTF;
extern double *TempdivTF;
extern double *TempelongTF;

extern double *OldARF;
extern double *OldAA;
extern double *OldBB;
extern double *OldCC;
extern double *OldDD;
extern double *OldPlt;
extern double *OldPltexpr;
extern double *OldPlts;
extern double *OldPltsexpr;
extern double *OldPP;
extern double *OldPin;
extern double *OldPin2;
extern double *OldAux1;
extern double *OldYuc3;
extern double *OldARR1;
extern double *OldARR1RGA;
extern double *OldARR12;

extern double *OldGh317;
extern double *OldGh33;
extern double *OlddivTF;
extern double *OldelongTF;
extern double *TempOlddivTF;
extern double *TempOldelongTF;
//cell based growth properties
extern int **OldSit;
extern int *MapCellNrs;
extern int *JustChecked;
extern int *TimePrevGrowth;
extern int *CellZonationState;
extern int *CellDivNr;
extern int *CellTimeLastDiv;
extern double *Diff;
extern double *Rdiv;
extern double *Relong;		
extern double *Rgrow;
extern double *Rgrowavg;
extern int *OldJustChecked;
extern int *OldTimePrevGrowth;
extern int *OldCellZonationState;
extern int *OldCellDivNr;
extern int *OldCellTimeLastDiv;
extern int *CellsInOZ;
extern int *OldCellsInOZ;
extern double *OldDiff;
extern double *OldRdiv;
extern double *OldRelong;
extern double *OldRgrow;
extern double *OldRgrowavg;
extern double *RandNr;

//extern int *Oscilationzone;
//auxin dynamics
extern double *a;
extern double *b;
extern double *c;
extern double *rhs;
extern double *res;
extern double **paux;
extern double **daux;
extern double ***Inf;
extern double ***Eff;
extern double ***DiffAuxC;
//graphics
extern unsigned char *RGBdata;

#endif
