/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef FunctionHeader
#define FunctionHeader

using namespace std;
//Main.cc
void MakeArrays();
void FreeArrays();

//Import.cc
void InitPlantFromFile(int start);

//Export.cc
void WritePlantToFile(int time);

//InitPlant.cc
void InitPlant();
void SetUpTissueLayout();
void NumberCells();
void FindCellCorners();
void FindCellSizes();
void SetUpMembraneTypes();

//Prepatterns.cc
void UpdateMembraneProteinsAndFluxes(int time);

//InitCells.cc
void InitConditions();

//Graphics.cc
double min(double a, double b);
double max(double a, double b);
void MakeGraphics(int t);
void MakePNGFile(int t,int flag);

//Plotting.cc
void ExtraMeasurements(int t);
void SaveTotAuxin(int t);
void SaveTotPlts(int t);
void TrackPltsSpaceTime(int t);
void SaveTemp(int t,int II,int JJ);
void SaveTempQC(int t);
void SaveGradCellularRes(int t);
void SaveEpidermalGrad(int time);
void SaveGradGridRes(int time);
void SaveLeftRightDiff(int t);
void TrackCellTypeSpaceTime(int t);
void TrackAuxSpaceTime(int t);
void Tracktotalmeasures(int t);

//Growthdecisions.cc
void maxmzsize();
int epidermalcell(int q);
void Growth(int time);
void GrowCell(int time,int q,int growthtype);

//Growthprocesses.cc
void DivideCell(int time,int q);
void Division(int time);
void CullOverflowCells(int time);
void CullLateralRootcap(int time);
void StoreOldSituation(int oldactcells);
void MapOldToNewSituation(int oldactcells);
void TrackCells(int time);

//Auxindiffusion.cc
void UpdateAuxin(int t);

//Update.cc //Cellexpression.cc
double positivesigmoid(double inputval,double km,double maxval=100);
double negativesigmoid(double inputval,double km,double maxval=100);

void UpdateGeneExpression(int time);


//Plasmodesmata.cc
void Plasmodesmatapatterning();

//Diffusion.cc
void CellularDiffusion(double hormonearray[], double maxvertdiffuse,double maxhordiffuse, bool MAINROOTFLUX=false,double hormoneshootlevel=0.0,double tspeedupstep=0.0);

//VisualiseFlux.cc
void StoreAndCompare(int time, int starttime);
void AuxinIncrease(int time); 
#endif
