/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef ConfigHeader
#define ConfigHeader

using namespace std;
#include "Basic_params.hh" //you cannot include defines for basic_params in Param_config.hh
#include "Plant_layout.hh" //you cannot include defines for plant_layout in Param_config.hh
#include "Auxin_params.hh" //you cannot include defines for auxin params in Param_config.hh


#include <iostream>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <png.h>
#include "dSFMT.h"

const double tstep=0.2;//0.2

const double tmax=2000000;//2419200;
const int maxnrsteps=(int)(tmax/tstep);
const int tsave=(int)(3*1200/tstep); //SHOULD BE 3600

const int speedup2=10;
const double tdelay=86400;
const int givenstart = 00000;




//#define RESTART

#define LRCLENGTHCONTROL
#define LATCRITERIUM
#define NOSIZELATCRITERIUM
#define LRCAUXINSENSITIVE

//#define AUTOACT //autoactivation of plethora
#define PLTDEPZONATION
#define JUNCTIONNOPLTS //no production of PLETHORA in the HCJ
#define JUNCTIONNOARR //no production of ARRs in the HCJ
//#define PLASMODESMATACLOSED
#define PLASMODESMATAOPEN
#define PLTLEAKAGETOMAINROOT
#define SELFARRINHIBIT

//#define STATICPLTS
//#define STATICARRS
const int arrversion= 1; //determines the type of static arr distribution. 0 for upwards from TZ, 1 for lowering level in DZ.
//#define SMBMUTANT
#define RGAARR1EFFECT
#define GROWTH

#define INITIALGROWTHSPREAD
#define DELAYEDGROWTH
#ifdef DELAYEDGROWTH
  const int initialgrowthtime= 16.*3600; //waiting time before growth starts to happen. Results in first division at 24 hpg
#endif

//GROWTH TYPE
//#define DISCRETEEXPONENTIALGROWTH
//#define DISCRETELINEARGROWTH
//#define RANDOMGROWTH
//#define RANDOMEXPANSION
//#define LINEARGROWTH
//#define LINEAREXPANSION
#define EXPONENTIALGROWTH
#define EXPONENTIALEXPANSION



#define GH317EPIDERMISONLY
#define DOUBLESLOWCELL
#define GH33FLOATVERSION //keep defined for proper save functionality

#define NORMALSHOOTROOTCONNECTION
//#define SHOOTROOTCUT 
//define LOWAUXIN_FROMSHOOT


#define JOINEDARRPINEFFS
#define PIN2ARRREGULATED
#ifdef JOINEDARRPINEFFS
const double PINARRSfac = 0.25;
const double ARRSKMPINS = 75;
#else
const double PINARR1fac = 0.7;
const double PINARR12fac= 0.3;
const double ARR1KMPINS = 150;
const double ARR12KMPINS = 40;
#endif 



//#define TROPIC
#ifdef TROPIC
//#define STATICGRAVITROPISM
//#define DYNAMICGRAVITROPISM
//#define ENHANCEDGRAVITROPISM
//#define HALOTROPIC
//#define ENHANCEDHALOTROPISM
#endif

//#define AUXINREGULATESPIN
#define PINREGULATED
//#define PINOUTWARDCORTEX
#define AUX1REGULATED
#define YUC3REGULATED
//#define PERICYCLEOUTWARD
#define EPIDERMALGROWTHCONTROL
#define POLAR
//#define APOLAR
//#define POLARMERISTEM
#define APOLARMERISTEM
//#define GRAVITROPIC
#define PIN1QCAPOLAR













//to get an idea of timing:
//d=0.0008;//(1/d)=1250s=20.8min=0.35hrs ln2/d=866s=14min=0.24hrs
//d=0.0002;//(1/d)=5000s=83min=1.39hrs ln2/d=3465s=58min=0.96hrs
//d=0.0001;//(1/d)=10000s=166min=2.78hrs ln2/d=6931s=115min=1.93hrs
//d=0.00005;//(1/d)=20000s=333min=5.55hrs ln2/d=13862s=231min=3.85hrs
//d=0.00002;//(1/d)=50000s=833min=13.89hrs ln2/d=34657s=577min=9.62hrs
//d=0.00001;//(1/d)=100000s=1666min=27.78hrs ln2/d=69314s=1155min=19.25hrs

//plt gradient depends on:
//at which auxin level half-saturation of expression 
//decay rate of plt, but this is also constrained 
//by time it takes for plt to become expressed
//maybe slow onset of plt expression should be 
//done using a delay time rather than using low
//production and decay rate, in this case upon
//cell growth expression level can be restored
//however it also means faster decay when cell
//is out of range expression domain, but maybe
//this makes sense: if cells undergo only few
//divisions rest of gradient is through decay
//amount of cellular growth, number of divisions
//spread through plasmodesmata


const double pDiv = 0.01;
const double dDiv = 0.0001;
const double PltsKM_Div = 40;
const double ARR1KM_Div= 25;
const double divisionthreshold = 40;

const double PltsKM_El = 25; 
const double baseElong = 0.25; //faction plts elongation repressed
const double ARRsKM_El = 80;
const double pEl =  0.01;
const double dEl = 0.0001;
const double elongtreshold = 40;


#define ARRELONGPLUSSED
#ifdef ARRELONGPLUSSED
 #define ARREFFECTPLUS
#endif
//#define ARRELONGEFFECTEMAX
#define DIVREPRESSELONG
#ifdef DIVREPRESSELONG
const double maxdivelong = .5;
const double DivKM_El = 30;
#endif



//#define ARRPINFLIP
#define EPIFLIPDIVISION // checks if ARR12's positive effect on the meristem is due to it being in control of when the epidermis flips.
#define LRCELONGREFLUX
#ifdef LRCELONGREFLUX
  #define LRCBACKELONG
#endif 
const double ARRpinboundary = 25;


const int PltsKM_PIN1 =80;//10;
const int PltsKM_PIN2 =80;//10;
const int AuxinKM_PIN1SHY2 =225;//10;//10
const int AuxinKM_PIN2SHY2 = 225;

const double offsetpin=0;
const double pPIN=0.01;
const double satpin=40;//Used only if there is a saturation of PIN export
const double dPIN=0.0001;
const double pinfac1=1.;//extent to which PIN1 regulation affects final PIN1 levels
const double pinfac2=1.;//extent to which PIN2 regulation affects final PIN2 levels
#define pPINSEPARATE
const double pPIN2=0.01;
const double dPIN2=0.0001;

extern double gauss[10000];
extern double tempcumgauss[100000];

//note that division takes only place in the straight part of the meristem
//we put in initially 30 cells, as cells will approximately be 1.5 their
//basic size this meanst that during growth on average 20 cells will be
//present
//given that the elongation zone is 7-11 cells and cells transit through it
//in approximately 7 hours, replenishment requires 1-1.5 cell divisions per
//meristematic cell file per hour
//assuming 30-35 actively dividing cells in the meristem this has led to
//estimations of division rates of 0.05-0.029 and hence of division times of
//20-35 hours
//however it seems that in the upper part of the meristem/TZ cells are still
//growing slowly but no longer dividing, meaning that less cells are 
//contributing to division and hence division times are overestimated
//relative to what they really are, if we for example assume that the last
//5 cells do not participate in division we get rates of 0.06-0.033 and hence
//times of 16.7-30 hrs, and numbers go down further if we take into account
//the much slower divisions closer to the SCN/QC
//in our model we assume that in the upper region size of 4 meristem cells
//this is the case, leaving us with 20-4=16 dividing cells
//this is much lower than in reality because the first 10 MZ cells are in the
//curved non dividing part
//as a consequence we need to lower our division times to get realistic 
//growth rates:
//1.5 cells/file/hr -> 1.5cells/16cells/hr=0.09375/hr -> 10.7 hrs
const double avgdivtime=10*3600+tmax;//6;//10*3600;//11*3600;//8.2*3600;//29520
const double avgelongtime=7*3600+tmax;//7.9*3600;//28440
const double varperc=0.05;//0.15;//0.025;//0.10;//0.15;//0.20;

//gene expression parameters
//GG
const double pGG=0.0065; //standin for pA,pB,pC and pD
const double dGG=0.000065; //standin for dA,dB,dC and dD

//#define ABDONLY //removes CC from the A to B to C to D to PLT, preserving more of the pattern.
const double ARFKM_A=55;
const double AKM_B=55;
const double BKM_C=55;
const double CKM_D=55;




//PLTslow 

const double halfAAPlts = 200; //Plts value for which auto activation is half max. Off by default
// If autoactivation is off, This value +1 is the total pPlts

const double auxAAweight = 4;// the extent to which autoactivation enhances Plethora self activation. 

#ifdef AUTOACT // off by default
  #define NEWAUTO
  #ifndef NEWAUTO
    const double halfARFAA = 75;
  #else
    const double halfAuxAA = 250;
  #endif
  const double pPlts=0.00175*2;//0.00175;//0.003;
#else
const double pPlts= 0.00175*(1+auxAAweight);//0.00175;//0.003;
#endif
const double Pltsdiff = 0.0000534;

const double satplts=60;
const double dPlts=0.0000175;//0.00003;//0.0000175;


//PLT
const double maxplt=0.00;
const double satplt=40;
const double decayplt=0.00018;//(1/d)=5000s=83min=1.39hrs ln2/d=3465s=58min=0.96hrs
const double offsetplt=0;


//ARR1
const double pARR1=0.02;
const double AuxKM_ARR1=160;
const double PltsKM_ARR1=35;
const double dARR1=0.0002;//0.00003;//0.0000175;
const double ARR1pltinhibitfrac= 0;
const double ARR1_self_max_inhibit = 0;
const double ARR1KM_self_inhibit = 300;
const double pARR1bind = 0.1;

//ARR12
const double pARR12=0.02;//0.00175;//0.003;
const double AuxKM_ARR12=175;
const double PltsKM_ARR12=25;	
const double dARR12=0.0002;//0.00003;//0.0000175;

const double ARR12pltinhibitfrac= 1;

//ARRS_combined
//#define ARRCOMBINEDPLTINHIBITION //turned off by default
#ifdef ARRCOMBINEDPLTINHIBITION
const double ARRsKM_Plts = 30; //unused version assuming ARR1 also does ARR12 repression and is combined with ARR12
#else
const double ARR1KM_Plts  = 35; //unused version assuming ARR1 also does ARR12 repression and is separate from ARR12
const double ARR12KM_Plts = 30; //unused version assuming ARR1 also does ARR12 repression and is separate from ARR12
#endif



//RGA
const double RGAARR1boundeff= 2;

//const int embryoGAtimelimit = 259200;//3*24*3600 ;//
const int embryoGAtimelimit = 3*24*3600;//24*3600;//givenstart+2*24*3600 ;//
const int reductiontime = 3*12*3600;//24*3600;
const double Embryoactivation = 0.7;


//Pin
const double pin1Pltsfac= 0.0;
const double pin2Pltsfac= 0.7;
//Yuc3
const double pYUC3=0.03;
const double PltsKM_YUC3=175;//20;//40;
const double dYUC3=0.0003;
const double yuc3fac=1.0;//0.9;//0.75;
const double maxyucAuxeff = 50;
const double pAyuc = maxyucAuxeff*pAux/100.;

#define GH33REGULATED
#ifdef GH33REGULATED
//Gh33
const double pGH33=0.03;
const double PltsKM_GH33=100;
const double dGH33=0.0003;
const double Gh33eff = 0.25;
const double dA_GH33 = Gh33eff* dAux/100.;
#endif
//Gh317
const double pGH317=0.03; 
const double dGH317=0.0003;

const double Gh317eff= 2.5;
const double dA_GH317 = Gh317eff*dAux/100.;
const double ARRsKM_GH317=75;



//Aux1
const double pAUX1=0.01;
const double dAUX1=0.0001;
const double AuxinKM_AUX1 =50;
#define AUX1PARTREGGED
#ifdef AUX1PARTREGGED
const double aux1_regfac = 1;
#endif
#define AUX1EPIVAL
#ifdef AUX1EPIVAL
const double aux1divzone = 0.1;
#endif
#define HIGHAUX1VASCMZ
#ifdef HIGHAUX1VASCMZ
const double vascmzaux1 = 2.;
#endif



#ifdef AUX1INEPI
const double aux1fac=0.07;
#else
const double aux1fac=0;
#endif
const double offsetaux1=0;


//Differentiation
const double decaydiffrate=0.00001*8;
const double diffrate=0.001*8;





#endif
