/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef StaticHeader
#define StaticHeader


using namespace std;
#include "Parameters.hh"
#include <iostream>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <png.h>
#include "dSFMT.h"



extern char dirname[10000];


//#define LAX3PREPATTERN
const double SatLax3 = 100;




 //#define CYTOKININ

extern double phase;


extern int idlist[3000];
extern int idcounter;
extern int memid;
extern int counter2;
extern int maxmzheight;
extern int rightsize;
extern int leftsize;
extern int totalrootcellnrL;
extern int totalrootcellnrR;
const double tstep=0.2;//0.2
const double tmax=2000000;//2419200;
const int maxnrsteps=(int)(tmax/tstep);
const int tsave=(int)(3*1200/tstep); //SHOULD BE 3600!!!!
const double xstep=1;//2;
const int speedup2=10;
const double tdelay=86400;
const int givenstart = 00000;

//parameters setting up horizontal tissue layout in straight part
const int disttolrc=20;//horizontal buffersize: start rectangle to start lrc
const int sizetoplrc=(int)(8./xstep)+1;//+1 is for wall
const int disttoepi=disttolrc+sizetoplrc;
const int nrepi=1;
const int sizeepi=(int)(18./xstep);//12
const int totsizeepi=nrepi*sizeepi+nrepi;
const int nrcort=1;
const int sizecort=(int)(20./xstep);//14
const int totsizecort=nrcort*sizecort+nrcort;
const int nrendo=1;
const int sizeendo=(int)(12./xstep);//8
const int totsizeendo=nrendo*sizeendo+nrendo;
const int nrperi=1;
const int sizeperi=(int)(8./xstep);//(int)(6./xstep)
const int totsizeperi=nrperi*sizeperi+nrperi;
const int nrvasc=3;//4
const int sizevasc=(int)(6./xstep);//(int)(4./xstep)
const int totsizevasc=nrvasc*sizevasc+nrvasc;
const int n=2*(disttolrc+sizetoplrc+totsizeepi+totsizecort+totsizeendo+totsizeperi+totsizevasc)+2-1;
const int xm=disttolrc+sizetoplrc+totsizeepi+totsizecort+totsizeendo+totsizeperi+totsizevasc;
//cell numbers
const int totcells=4000;                            //maximum totalnr of cells
extern int actcells;
const int zoom=1;               //zoom factor used for display



//parameters setting up vertical tissue layout
//cell sizes
const int HS1=44/xstep;//height first columella cell
const int HS2=30/xstep;//height second columella cell
const int HS3=20/xstep;//height third columella cell
const int HS4=14/xstep;//height fourth columella cell
const int HS5=10/xstep;//height fifth columella cell
const int HCI=8/xstep;//height columella initial
const int HMZ=8/xstep;//!!!!!!!!!!10/xstep;//height meristem cells
const int Hndivi =18/xstep;
const int HEZ=60/xstep;//height elongation zone cells GRADIENT OF DIFFERENT SIZES USED
const int HDZ=100/xstep;//height differentiation zone cells DIFFERENT SIZES USED
const int HJZ=60/xstep; //height hypocotyljunction cells


const int Hdiv=2*HMZ;
const int Hdivlrc=4*HMZ;
const int Hmax=144/xstep;
// EZ and DZ cell heights:


//vertical start positions
const int yzero=8;                              //bottom buffer size, start rectangle to root tip 
const int yQC=yzero+(HS1+HS2+HS3+HS4+HS5+HCI)+6;//bottom position QC: yzero+6 cells columella +6 walls
const int yR=yQC+3*HMZ+3; //yQC+5*HMZ+5;    //start radial lines: yQC +5 MZ cells+5 walls
const int yLRCstraight=yR-2*HMZ-2;//const int yLRCstraight=yR;//yR+5*HMZ+5;
const int yDivi = yLRCstraight+nrcellsstraightMZ*HMZ+nrcellsstraightMZ;
const int yMZEZ=yDivi+nrcellsNDIV*Hndivi+nrcellsNDIV;//MZ-EZ boundary MZ supposedly 30 cells
const int yLRC1=yDivi+nrcellsNDIV*Hndivi+nrcellsNDIV;//typically equal to yMZEZ
const int yEND=HMZ*40+Hndivi*10+6*HEZ+10*HDZ+10;         //15 cells (10 EZ 5 DZ) + 16 walls  
const int TOPJCELLS = yMZEZ+nrcellsEZ*HEZ/2+nrcellsDZ*HDZ+HJZ*nrcellsHJ+nrcellsEZ+nrcellsDZ+nrcellsHJ+1;
const int m=(int)(yEND+30);                   //size of field in y-dir 




//how to construct a parabola: y=ax^n+b
//we know two points, say (xLRC2,yLRC2) which is the upperleft startpoint 
//and the point (xm,yzero+HS1+1) which is the minimum of the parabola 
//if we now redefine x as x=abs(x-xm) these points become
//translated into (abs(xLRC2-xm),yLRC2) and (0,yzero+HS1+1)
//for x=0 y=b thus b=yzero+HS1+1 is the y value of this minimum point
//the value of a then follows from a=(y-b)/x^n

//parabola lines
//parabola around LRC1
const int x0LRC1=disttolrc;//-2/xstep;
const double bLRC1=yzero;
const int powLRC1=3;
const double aLRC1=(yLRCstraight-bLRC1)/(abs(pow((x0LRC1-xm),powLRC1))); //!
//so for y<yLRC1: y=aLRC1*(x-xm)^2+bLRC1

//parabola around LRC2
const int x0LRC2=x0LRC1+16/xstep+1;//+12/xstep+1;
const int yLRC2=yQC-10/xstep;//+10/xstep;//+6*HMZ+2/xstep;//-4 voor xstep=1
const double bLRC2=bLRC1+HS1+1; 
const int powLRC2=3;
const double aLRC2=(yLRC2-bLRC2)/(abs(pow((x0LRC2-xm),powLRC2)));
//so for y<yLRC2: y=aLRC2*(x-xm)^2+bLRC2

//parabola around LRC3
const int x0LRC3=x0LRC2+16/xstep+1;//x0LRC2+16/xstep+1;
const int yLRC3=yQC-(30/xstep);//yQC-(20/xstep);//+2*HMZ+(2/xstep);//yQC+2*HMZ+(6/xstep);//+3 voor xstep=1
const double bLRC3=bLRC2+HS2+1;
const int powLRC3=3;
const double aLRC3=(yLRC3-bLRC3)/(abs(pow((x0LRC3-xm),powLRC3)));
//so for y<yLRC3: y=aLRC3*(x-xm)^2+bLRC3

//parabola around LRC4
const int x0LRC4=x0LRC3+4/xstep+1;//x0LRC3+14/xstep+1;
const int yLRC4=yQC-(26/xstep);//yQC-(12/xstep);//-(8/xstep);//yQC-(4/xstep);
const double bLRC4=bLRC3+HS3+1;
const int powLRC4=2;
const double aLRC4=(yLRC4-bLRC4)/(abs(pow((x0LRC4-xm),powLRC4)));
//so for y<yLRC4: y=aLRC4*(x-xm)^2+bLRC4


//note that what is called intercept here is the same as the
//x... in the above parabolas: the x coordinate of the left
//upper intersection point of the parabola;
//note that for all parabola the corresponding y value of this
//point should be yLRCstraight, below which the parabolas start
//parabola around epi and underneath S5
const int intercEPI=disttoepi;
const double bEPI=bLRC4+HS4-1;//+1;
const int powEPI=3;
const double aEPI=(yLRCstraight-bEPI)/(abs(pow((intercEPI-xm),powEPI)));
//so for y<yLRC1 y=aEPI*(x-xm)^2+bEPI

//parabola around cortex and underneath QC
const int intercCORT=intercEPI+totsizeepi;
const double bCORT=bEPI+HS5+1;
const int powCORT=3;
const double aCORT=(yLRCstraight-bCORT)/(abs(pow((intercCORT-xm),powCORT)));
//so for y<yLRC1 y=aCORT*(x-xm)^3+bCORT

//parabola around endo and underneath QC (needs to stop early)
const int intercENDO=intercCORT+totsizecort;
const double bENDO=bCORT+0.6*HCI+1;
const int powENDO=4;
const double aENDO=(yLRCstraight-bENDO)/(abs(pow((intercENDO-xm),powENDO)));
//so for y<yLRC1 y=aENDO*(x-xm)^4+bENDO

//parabola around peri and at bottom QC 
const int intercPERI=intercENDO+totsizeendo;
const double bPERI=yQC;
const int powPERI=5;
const double aPERI=(yLRCstraight-bPERI)/(abs(pow((intercPERI-xm),powPERI)));
//so for y<yLRC1 y=aPERI*(x-xm)^4+bPERI

//parabola around vasc and at bottom QC
const int intercVASC=intercPERI+totsizeperi;
const double bVASC=yQC;
const int powVASC=3;//4;
const double aVASC=(yLRCstraight-bVASC)/(abs(pow((intercVASC-xm),powVASC)));
//so for y<yLRC1 y=aVASC*(x-xm)^4+bVASC
 
//radial lines
//all go through point x=xm, y=yR
//equation for lines should be of form y=a*x+b
//so we know alreadu yR=a*xm+b
//so pick desired slope a and find b
const double arad1=0.5;
const double brad1=yR-arad1*xm;
const double arad2=1;//1.2;//1.3;//CHANGED!!!!
const double brad2=yR-arad2*xm;
const double arad3=2.1;//2.3;//2.5;;//CHANGED!!!!
const double brad3=yR-arad3*xm;
const double arad4=5;//6;
const double brad4=yR-arad4*xm;





//to get an idea of timing:
//d=0.0008;//(1/d)=1250s=20.8min=0.35hrs ln2/d=866s=14min=0.24hrs
//d=0.0002;//(1/d)=5000s=83min=1.39hrs ln2/d=3465s=58min=0.96hrs
//d=0.0001;//(1/d)=10000s=166min=2.78hrs ln2/d=6931s=115min=1.93hrs
//d=0.00005;//(1/d)=20000s=333min=5.55hrs ln2/d=13862s=231min=3.85hrs
//d=0.00002;//(1/d)=50000s=833min=13.89hrs ln2/d=34657s=577min=9.62hrs
//d=0.00001;//(1/d)=100000s=1666min=27.78hrs ln2/d=69314s=1155min=19.25hrs

//grid based properties
extern double **Caux1;
extern double **Caux2;
extern double **Cauxint;
#ifdef SATURATION  
extern double **inf_sat;
extern double **eff_sat;
extern double **pPUMP_sat;
extern double **aux1fac_sat;
extern double **lax3fac_sat;
#endif
extern double **Plasmodesmatapattern;
extern int **Tc;
extern int **Tmt;
extern int **Tct;
extern double **PinPrePat;
extern double **Pin2PrePat;
extern double **Cpin1M;
extern double **Cpin2M;
extern double **Aux1PrePat;
extern double **Aux1M;
extern double **Lax3PrePat;
extern double **Lax3M;
extern double **Pltsgrid;
extern double **Pltsgrid2;
extern double **dummypict;
extern double *BaseMeanCell;
extern double *TempAuxin;
extern double *IncrAuxin;
extern bool *Auxminpassed;
extern bool *OldAuxminpassed;
extern bool *ARRpassed;
extern bool *OldARRpassed;
//cells on grid
extern int **Cell;
//cell based properties
extern int *CellMinI;
extern int *CellMaxI;
extern int *CellMinJ;
extern int *CellMaxJ;
extern int *CellWidth;
extern int *CellHeight;
extern int *CellSizes;
extern double *InvCellSizes;
extern double *MeanCells;
extern double *OldMeanCells;
extern double *BaselineAuxine;
extern double *DiffAuxine;
extern double *TimeOfAuxineIncrease;
//cell based gene expression
extern double *ARF;
extern double *AA;
extern double *BB;
extern double *CC;
extern double *DD;
extern double *Plt;
extern double *Pltexpr;
extern double *Plts;
extern double *Pltsexpr;
extern double *PP;
extern double *Pin;
extern double *Pin2;
extern double *Aux1;
extern double *Lax3;
extern double *Yuc3;
extern double *ARR1;
extern double *ARR1RGA;
extern double *ARR12;

extern double *Gh317;
extern double *Gh33;
extern double *divTF;
extern double *elongTF;
extern double *TempdivTF;
extern double *TempelongTF;

extern double *OldARF;
extern double *OldAA;
extern double *OldBB;
extern double *OldCC;
extern double *OldDD;
extern double *OldPlt;
extern double *OldPltexpr;
extern double *OldPlts;
extern double *OldPltsexpr;
extern double *OldPP;
extern double *OldPin;
extern double *OldPin2;
extern double *OldAux1;
extern double *OldLax3;
extern double *OldYuc3;
extern double *OldARR1;
extern double *OldARR1RGA;
extern double *OldARR12;

extern double *OldGh317;
extern double *OldGh33;
extern double *OlddivTF;
extern double *OldelongTF;
extern double *TempOlddivTF;
extern double *TempOldelongTF;
//cell based growth properties
extern int **OldSit;
extern int *MapCellNrs;
extern int *JustChecked;
extern int *TimePrevGrowth;
extern int *CellZonationState;
extern int *CellDivNr;
extern int *CellTimeLastDiv;
extern double *Diff;
extern double *Rdiv;
extern double *Relong;		
extern double *Rgrow;
extern double *Rgrowavg;
extern int *OldJustChecked;
extern int *OldTimePrevGrowth;
extern int *OldCellZonationState;
extern int *OldCellDivNr;
extern int *OldCellTimeLastDiv;
extern int *CellsInOZ;
extern int *OldCellsInOZ;
extern double *OldDiff;
extern double *OldRdiv;
extern double *OldRelong;
extern double *OldRgrow;
extern double *OldRgrowavg;
extern double *RandNr;

//extern int *Oscilationzone;
//auxin dynamics
extern double *a;
extern double *b;
extern double *c;
extern double *rhs;
extern double *res;
extern double **paux;
extern double **daux;
extern double ***Inf;
extern double ***Eff;
extern double ***DiffAuxC;
//graphics
extern unsigned char *RGBdata;




extern dsfmt_t dsfmt;
inline double uniform() { return dsfmt_genrand_close_open(&dsfmt); }



//Main.cc
void MakeArrays();
void FreeArrays();

//Import.cc
void InitPlantFromFile(int start);

//Export.cc
void WritePlantToFile(int time);

//InitPlant.cc
void InitPlant();
void SetUpTissueLayout();
void NumberCells();
void FindCellCorners();
void FindCellSizes();
void SetUpMembraneTypes();

//Prepatterns.cc
void SetUpPrePattern();

//InitCells.cc
void InitConditions();

//Graphics.cc
double min(double a, double b);
double max(double a, double b);
void MakeGraphics(int t);
void MakePNGFile(int t,int flag);

//Plotting.cc
void ExtraMeasurements(int t);
void SaveTotAuxin(int t);
void SaveTotPlts(int t);
void TrackPltsSpaceTime(int t);
void SaveTemp(int t,int II,int JJ);
void SaveTempQC(int t);
void SaveGradCellularRes(int t);
void SaveEpidermalGrad(int time);
void SaveGradGridRes(int time);
void SaveLeftRightDiff(int t);
void TrackCellTypeSpaceTime(int t);
void TrackAuxSpaceTime(int t);
void Tracktotalmeasures(int t);

//Growthdecisions.cc
void maxmzsize();
int epidermalcell(int q);
void Growth(int time);
void GrowCell(int time,int q,int growthtype);

//Growthprocesses.cc
void DivideCell(int time,int q);
void Division(int time);
void CullOverflowCells(int time);
void CullLateralRootcap(int time);
void StoreOldSituation(int oldactcells);
void MapOldToNewSituation(int oldactcells);
void TrackCells(int time);

//Auxindiffusion.cc
void UpdateAuxin(int t);

//Update.cc //Cellexpression.cc
double positivesigmoid(double inputval,double km,double maxval=100);
double negativesigmoid(double inputval,double km,double maxval=100);

void UpdateGeneExpression(int time);
void UpdateMembraneProteinsAndFluxes(int time);


//Plasmodesmata.cc
void Plasmodesmatapatterning();

//Diffusion.cc
void CellularDiffusion(double hormonearray[], double maxvertdiffuse,double maxhordiffuse, bool MAINROOTFLUX=false,double hormoneshootlevel=0.0,double tspeedupstep=0.0);

//VisualiseFlux.cc
void StoreAndCompare(int time, int starttime);
void AuxinIncrease(int time); 

#endif

