/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "Arrays.hh"
#include "Auxin_params.hh"
#include "Basic_params.hh"
#include "Functions.hh"
#include "Param_config.hh"
#include "Plant_layout.hh"

void CellularDiffusion(double hormonearray[], double maxvertdiffuse,double maxhordiffuse, bool MAINROOTFLUX,double hormoneshootlevel,double tspeedupstep){
    //tmt
    //1 up
    //2 down
    //3 in
    //4 out

  double hormonegrid1[n][m];
  double hormonegrid2[n][m];

  int Hdiv=2*(HMZ+1);
  for(int i=0;i<n;i++){
    for(int j=0;j<m;j++){ 
	  if(Cell[i][j]!=-1){

	    int q=Cell[i][j];
	    hormonegrid1[i][j]=hormonearray[q];
  	    hormonegrid2[i][j]=hormonearray[q];

      }
	  else{
	    hormonegrid1[i][j]=0;
	  }
    }
  }

 double vertdiffuse = maxvertdiffuse;
 double latdiffuse = maxhordiffuse;
 for(int i=0;i<n;i++){
    for(int j=0;j<m;j++)
      {
      
	  vertdiffuse = maxvertdiffuse;
	  	  latdiffuse = maxhordiffuse;
	  
      if(Tc[i][j]==2)//mebrane, so point of possible exchange
	    {
	    int currentcell = Cell[i][j];
        int membranelookup = Tmt[i][j] - 1;
	    if(Tc[i-1][j]==0 )//wall to left, so left membrane
	      {
		int ii=i-1;
		while(ii>0 && (i-ii)<4)
		  {
		    ii--;
		    if(Tc[ii][j]==2)//neighbouring membrane found
		      {
			    hormonegrid2[i][j]+= 0.5*Plasmodesmatapattern[currentcell][membranelookup]*latdiffuse*(hormonegrid1[ii][j]-hormonegrid1[i][j]);
			    hormonegrid2[ii][j]+= 0.5*Plasmodesmatapattern[currentcell][membranelookup]*latdiffuse*(hormonegrid1[i][j]-hormonegrid1[ii][j]);
             break;
            }
          }
	      }
	    if(Tc[i+1][j]==0)//wall to right, so right membrane
	      {
		int ii=i+1;

		while(ii<n && (ii-i)<4)
		  {
		    ii++;
		    if(Tc[ii][j]==2)//neighbouring membrane found
		      {
			hormonegrid2[i][j]+= 0.5*Plasmodesmatapattern[currentcell][membranelookup]*latdiffuse*(hormonegrid1[ii][j]-hormonegrid1[i][j]);
			hormonegrid2[ii][j]+= 0.5*Plasmodesmatapattern[currentcell][membranelookup]*latdiffuse*(hormonegrid1[i][j]-hormonegrid1[ii][j]);
			break;
		      }
		  }
	      }
	    if(Tc[i][j-1]==0 )//wall above, so apical membrane ???????
	      {
		int jj=j-1;

		while(jj>0 && (j-jj)<4)
		  {
		    jj--;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			hormonegrid2[i][j]+=0.5*Plasmodesmatapattern[currentcell][membranelookup]*vertdiffuse*(hormonegrid1[i][jj]-hormonegrid1[i][j]);
			hormonegrid2[i][jj]+=0.5*Plasmodesmatapattern[currentcell][membranelookup]*vertdiffuse*(hormonegrid1[i][j]-hormonegrid1[i][jj]);
			break;
		      }		 
		  }
	      }
	    if(Tc[i][j+1]==0)//wall below, so basal membrane ?????????
	      {
		int jj=j+1;

		while(jj<m && (jj-j)<4)
		  {
		    jj++;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			hormonegrid2[i][j]+=0.5*Plasmodesmatapattern[currentcell][membranelookup]*vertdiffuse*(hormonegrid1[i][jj]-hormonegrid1[i][j]);
			hormonegrid2[i][jj]+=0.5*Plasmodesmatapattern[currentcell][membranelookup]*vertdiffuse*(hormonegrid1[i][j]-hormonegrid1[i][jj]);
			break;
		      }		 
		  }
	      }
	  }
      }
  }
  if(MAINROOTFLUX){
  double Pltslevelshoot=0;
  double pdfluxshoot=maxvertdiffuse;
  for(int i=0;i<n;i++){
    for(int j=m-1;j>=0;j--)
      {
	if(Cell[i][j]!=-1)//topmost part of topmost cell found
	  {
	    hormonegrid2[i][j]+=pdfluxshoot*(hormoneshootlevel-hormonegrid1[i][j]);
	    break;
	  }
      }
  }
}
bool COLUMMELLADEGR=true;
  if(COLUMMELLADEGR){
    double peeldegrrate = 0.5/avgdivtime;

	for (int i = 0;i<n;i++){
	  for (int j =0; j<yQC;j++){
        if (Tct[i][j]==13||Tct[i][j]==1){
		  hormonegrid2[i][j]-= peeldegrrate * hormonegrid1[i][j];
		}
    }
  }
}

bool mimicsidegrowth= false;
if (mimicsidegrowth){
  for (int j = yQC; j < yLRCstraight+4; j++){
    for (int i = 0; i < n; i++){
      if(Tc[i][j]==2){//mebrane, so point of possible exchange
	     if(Tc[i-1][j]==0 && i< xm-6){//wall to left, so left membrane
          int ii=i-1;
          while(ii>0 && (i-ii)<4){
            ii--;
		      if(Tc[ii][j]==2){//neighbouring membrane found
			     hormonegrid2[i][j]-=tspeedupstep*(1.0/avgdivtime *0.5)*(CellMaxJ[Cell[i][j]]-CellMinJ[Cell[i][j]])*hormonegrid1[i][j]; //losing to outside
			     hormonegrid2[ii][j]+=tspeedupstep*(1.0/avgdivtime *0.5)*(CellMaxJ[Cell[i][j]]-CellMinJ[Cell[i][j]])*hormonegrid1[i][j]; //gaining from inside
              break;
              }
            }
          }
        if (Tc[i+1][j]==0 && i>xm+6){//wall to right, so right membrane
		int ii=i+1;
          while(ii<n && (ii-i)<4){
            ii++;
		      if(Tc[ii][j]==2){//neighbouring membrane found
			     hormonegrid2[i][j]-=tspeedupstep*(1.0/avgdivtime *0.5)*(CellMaxJ[Cell[i][j]]-CellMinJ[Cell[i][j]])*hormonegrid1[i][j]; //losing to outside
			     hormonegrid2[ii][j]+=tspeedupstep*(1.0/avgdivtime *0.5)*(CellMaxJ[Cell[i][j]]-CellMinJ[Cell[i][j]])*hormonegrid1[i][j]; //gaining from inside

              break;
              }
            }
          }
        }
      }
    }
  }

	 
	 

  //transform back from grid to cellular
  //assumption that within cell PLTs differences
  //even out quickly through diffusion
  for(int q=0;q<totcells;q++){
    hormonearray[q]=0;
  }
  for(int i=0;i<n;i++){
    for(int j=0;j<m;j++){
    	if(Cell[i][j]!=-1){//cell there
 	     int q=Cell[i][j];
        hormonearray[q]+=hormonegrid2[i][j];
	   }
    }
  } 
  for(int q=0;q<actcells;q++){

    hormonearray[q]*=(double)InvCellSizes[q];
    if (Tct[CellMinI[q]+1][CellMinJ[q]+1]==12){
      }


  }



} 
	


