/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "Arrays.hh"
#include "Auxin_params.hh"
#include "Basic_params.hh"
#include "Functions.hh"
#include "Param_config.hh"
#include "Plant_layout.hh"




void maxmzsize(){
  int i = disttoepi+1;
  for (int j= yLRCstraight;j<m;j++){
    if(Cell[i][j]!=-1){
      int q = Cell[i][j];
      if (CellZonationState[q] >50){
        maxmzheight = j-2; 
        break;
      }
    }
  }
}

bool growtiming(int time, int q,int growtype)
{
bool grown = false;
#ifdef RANDOMGROWTH
		  if((100*RandNr[CellMinJ[q]])<(100*0.000198))
		    {
		      TimePrevGrowth[q]=time;
		      GrowCell(time,q,growtype);
            grown = true;
		      q=-1;
		    }
#endif
#ifdef EXPONENTIALGROWTH


		  double tgrowth=1./(Rgrow[q]*CellHeight[q]);	      
		  if(time>=TimePrevGrowth[q]+tgrowth)
		    {
		      TimePrevGrowth[q]=time;
		      GrowCell(time,q,growtype);
            grown = true;

		    } 
#endif
#ifdef LINEARGROWTH
		  tgrowth=1./(Rgrow[q]);
		  if(time>=TimePrevGrowth[q]+tgrowth)
		    {
		      TimePrevGrowth[q]=time;
		      GrowCell(time,q,growtype);
            grown = true;

		    } 
#endif	 
return grown;
}










bool checkauxinminimum(int checkcell){

  #ifdef AUXMINVAL
  if(MeanCells[checkcell]<auxinminimum){
    return true;
  }
  else {
    return false;
  }
  
  #elif defined AUXMINFREQ
  int horizonmid = (CellMinI[checkcell]+CellMaxI[checkcell])/2;
  int height=CellMaxJ[checkcell];
  int uppercell = Cell[horizonmid][height]; 
  while(uppercell == -1 || uppercell == checkcell ){
    uppercell = Cell[horizonmid][height]; 
    height++;
  }
  height=CellMinJ[checkcell];
  int lowercell = Cell[horizonmid][height]; 
  while(lowercell == -1 || lowercell == checkcell ){
    lowercell = Cell[horizonmid][height]; 
    height--;
  }
  
  if ( MeanCells[lowercell]> auxminfrac*MeanCells[checkcell] && MeanCells[uppercell]> auxminfrac*MeanCells[checkcell]){
    return  true;
  }
  else {
    return false;
  }
  #else 
    cout << "invalid combination, please decide how the auxin minimum should be made." << std::endl;
    exit(1);
  #endif
}
/* function that makes sure cell movement doesn't cause pattern breaks*/
void correcttop(){ 
  int regulardrop = 101;
  int previousmaxj = 0; //used to see if the distance between two cells exceeds the regular drop it checks on.
  for (int i= 0;i<n/2-1;i++){
    for (int j= m-2;j>=regulardrop+1;j--){
      if (Tc[i][j]!=3){      
        for (int jj= j; jj>=j-regulardrop;jj--){
          int iplus = 0;
          int idirect = 1;
          if  (Tc[i+1][jj]==0 && ((Tct[i][jj]>0 && Tct[i][jj]<6)||(Tct[i][jj-1]>0 && Tct[i][jj-1]<6) ) ){ // add in extra check of cell wall to the side for lateral root cap         
            idirect = 1;
            iplus =2;
            }
          else if  (Tc[i-1][jj]==0  && ((Tct[i][jj]>0 && Tct[i][jj]<6) || (Tct[i][jj-1]>0 && Tct[i][jj-1]<6))  ){
            idirect = -1;
            iplus = 2;
            }
          else{
            iplus = 1;
            idirect = 1;
            }
          for (int iter = 0; iter <=iplus; iter++){
            int ifirst = i+iter*idirect;
            int ii = n-1-(ifirst);
            if (  int(1000*Caux1[ifirst][jj])!=int(1000*Caux1[ii][jj])            ||
              int(1000*PinPrePat[ifirst][jj])!=int(1000*PinPrePat[ii][jj])    ||
              int(1000*Pin2PrePat[ifirst][jj])!=int(1000*Pin2PrePat[ii][jj])    ||
              int(1000*Aux1PrePat[ifirst][jj])!=int(1000*Aux1PrePat[ii][jj])  ||
              Tc[ii][jj]!=Tc[ifirst][jj] ||Tct[ii][jj]!=Tct[ifirst][jj]  ){

              Tc[ii][jj]=Tc[ifirst][jj];
	           Tct[ii][jj]=Tct[ifirst][jj];
	           Tmt[ii][jj]=Tmt[ifirst][jj];
              //shift Pin/Aux1
              PinPrePat[ii][jj]=PinPrePat[ifirst][jj];
              Pin2PrePat[ii][jj]=Pin2PrePat[ifirst][jj];
          	  Aux1PrePat[ii][jj]=Aux1PrePat[ifirst][jj];
              //shift auxin levels
              Caux1[ii][jj]=Caux1[ifirst][jj];

            }
          }
        }
        if (j >=previousmaxj+regulardrop){ //you moved up a lot more in this row than the last
          for (int jj = previousmaxj-2; jj <= j-regulardrop+1;jj++){
            int ii = n-1-i;
            if (  int(1000*Caux1[i][jj])!=int(1000*Caux1[ii][jj])            ||
              int(1000*PinPrePat[i][jj])!=int(1000*PinPrePat[ii][jj])    ||
              int(1000*Pin2PrePat[i][jj])!=int(1000*Pin2PrePat[ii][jj])    ||
              int(1000*Aux1PrePat[i][jj])!=int(1000*Aux1PrePat[ii][jj])  ||
              Tc[ii][jj]!=Tc[i][jj] ||Tct[ii][jj]!=Tct[i][jj]  ){
              cout << "attempted prevention at ii " << ii << " jj " << jj << std::endl;
              Tc[ii][jj]=Tc[i][jj];
	           Tct[ii][jj]=Tct[i][jj];
	           Tmt[ii][jj]=Tmt[i][jj];
              //shift Pin/Aux1
              PinPrePat[ii][jj]=PinPrePat[i][jj];
              Pin2PrePat[ii][jj]=Pin2PrePat[i][jj];
          	  Aux1PrePat[ii][jj]=Aux1PrePat[i][jj];
              //shift auxin levels
              Caux1[ii][jj]=Caux1[i][jj];
               
            }
          }        
        }
        else if (j+regulardrop <= previousmaxj){ //the cellfile we are now in has dropped more than the previous file looked down to correct itself
          int icheck = i-1; //we're interested in the wall to the left
          int ii = n-1-icheck;
          
          for (int jj = j-2; jj <= previousmaxj-regulardrop;jj++){
            if (  int(1000*Caux1[icheck][jj])!=int(1000*Caux1[ii][jj])            ||
              int(1000*PinPrePat[icheck][jj])!=int(1000*PinPrePat[ii][jj])    ||
              int(1000*Pin2PrePat[icheck][jj])!=int(1000*Pin2PrePat[ii][jj])    ||
              int(1000*Aux1PrePat[icheck][jj])!=int(1000*Aux1PrePat[ii][jj])  ||
              Tc[ii][jj]!=Tc[icheck][jj] ||Tct[ii][jj]!=Tct[icheck][jj]  ){
              cout << "attempted prevention at ii " << ii << " jj " << jj  << std::endl;
              Tc[ii][jj]=Tc[icheck][jj];
	           Tct[ii][jj]=Tct[icheck][jj];
	           Tmt[ii][jj]=Tmt[icheck][jj];
              //shift Pin/Aux1
              PinPrePat[ii][jj]=PinPrePat[icheck][jj];
              Pin2PrePat[ii][jj]=Pin2PrePat[icheck][jj];
          	  Aux1PrePat[ii][jj]=Aux1PrePat[icheck][jj];
              //shift auxin levels
              Caux1[ii][jj]=Caux1[icheck][jj];
               
            }
          } 




        }

        previousmaxj=j;
        break;
      }
    }
  }

}

/* finds the epidermal cell next to the middle of the given input cell.*/
int epidermalcell(int q){
  int epiIval = n-disttoepi-2;
  
  //currently turned off right/left asymetry. If we want to turn it on we will have to grow from the inside out,
  //or the cells will act differently when the epidermis on the left has divided and the right has not had its turn yet.
  if (CellMinI[q]<n/2-1){ //check left epidermis
	epiIval = disttoepi+1; //+1 to make sure you are inside the cell
  }
  if (CellMaxI[q]>n/2-1){ //check right epidermis
	epiIval = n-disttoepi-2; //n-1 because arrays are from 0 and -1 again to be sure you are inside the cell
  }	  
  
  int epidermalcell = Cell[epiIval][((CellMaxJ[q]+CellMinJ[q])/2)];
 
  for(int iter = 0; epidermalcell ==-1;iter++){
    epidermalcell = Cell[epiIval][((CellMaxJ[q]+CellMinJ[q])/2)-iter];
  }
  if( Tct[epiIval][CellMaxJ[epidermalcell]-1] != 5){
	cout << "epidermis not properly located " <<std::endl;
	exit(1);
  }
  return epidermalcell;
}
int cortexcell(int q){
  int corIval = n-disttoepi-totsizeepi-2;
 
  //currently turned off right/left asymetry. If we want to turn it on we will have to grow from the inside out,
  //or the cells will act differently when the epidermis on the left has divided and the right has not had its turn yet.
  if (CellMinI[q]<n/2-1){ //check left epidermis
	corIval = disttoepi+totsizeepi+2;; //+1 to make sure you are inside the cell
  }
  if (CellMaxI[q]>n/2-1){ //check right epidermis
    corIval = n-disttoepi-totsizeepi-2;; //n-1 because arrays are from 0 and -1 again to be sure you are inside the cell
  }	  
  
  int cortexcell = Cell[corIval][((CellMaxJ[q]+CellMinJ[q])/2)];
  for(int iter = 0; cortexcell ==-1;iter++){
    cortexcell = Cell[corIval][((CellMaxJ[q]+CellMinJ[q])/2)-iter];
  }
  if( Tct[corIval][CellMaxJ[cortexcell]-1] != 6){
	cout << "cortex not properly located " << " Tct "<< Tct[corIval][CellMaxJ[cortexcell]-1] <<std::endl;
	exit(1);
  }

  return cortexcell;
}

double checkavevalue(int j, int checktype, double valuearray[]){
  int i;
  if (checktype ==0) i = disttoepi+1;
  else if  (checktype ==1) i = disttolrc+sizetoplrc+totsizeepi+totsizecort+totsizeendo+totsizeperi+1;
  int checkq= Cell[i][j];
  int cellnumber=1;
  double returnvalue = valuearray[checkq];
  for (i;i < n/2;i++){
    if (Cell[i][j]!=-1 && Cell[i][j]!=checkq){
      checkq=Cell[i][j];
      cellnumber++;
      returnvalue += valuearray[checkq];
    }
  }
  return returnvalue/cellnumber;
}

 



void Growth(int time)
{
  int i,j,q;
  int H;
  double P;
  int meani,mini,maxi,meanj;
  int tgrowth;
  static int first_time=0;
  bool grown = false;
  bool divided = 0;
  bool growcheck = false;



  #ifndef GROWTHCHECK
  int checktype = 4; // determines how you want to calculate values for growth. Put this in Header.hh.
  #endif
 
#ifndef RANDOMGROWTH
  double divtime;
  double elongtime;
  double sumdiv;
  double divrate;
  double sumelong;
  double elongrate;
  double steptime;
  double R1;
  if(first_time==0)
    {
     
      for(q=0;q<actcells;q++)
	{

  #ifdef EPIDERMALGROWTHCONTROL
    int checkcell = q;
    if (CellMinJ[q]>yLRCstraight){
      checkcell = epidermalcell(q);
    }
  #else
    int checkcell = q;
  #endif	

     #ifndef DOUBLESLOWCELL
     if((CellMinJ[q] <= (yLRCstraight + 2)) &&  (CellMinJ[q] >= yLRCstraight)){
       divtime=avgdivtime+int(avgdivtime*.5);
     }
	  #else
     if(CellDivNr[q] ==0&& Tct[CellMinI[q]+1][CellMinJ[q]+1]>4){
       divtime=avgdivtime+int(avgdivtime*1.05);
       #ifdef FASTSCNDIV
         divtime *= (1./scndivmod);
       #endif
     }
     else if(CellDivNr[q] ==1 && Tct[CellMinI[q]+1][CellMinJ[q]+1]>4){
       divtime=avgdivtime+int(avgdivtime*.45);
       #ifdef FASTSCNDIV
         divtime *= (1./scndivmod);
       #endif
     } 
     #endif
     else {		 
       divtime=avgdivtime;
		 #ifdef FASTRAPDIV
		   divtime *= (1./rapdivmod);
       #endif
     }
     

#ifdef EXPONENTIALGROWTH
	  sumdiv=0;
	  for(i=HMZ;i<=2*HMZ;i++)
	    sumdiv+=(1./((double)i));
	  divrate=sumdiv/divtime;
	  Rdiv[q]=divrate;
#endif
#ifdef LINEARGROWTH
	  steptime=divtime/HMZ;
	  divrate=1./steptime;
	  Rdiv[q]=divrate;
#endif
	  elongtime=avgelongtime;
#ifdef EXPONENTIALEXPANSION
	  sumelong=0;
	  for(i=((int)HMZ);i<=Hmax;i++)
	    sumelong+=(1./((double)i));
	  elongrate=sumelong/elongtime;
  #ifndef ARRELONGSPECIFIED
     double baseElong= 1;
     double arr12Elong = 0.0;
     double arr1Elong = 0.0;
     double maxElong = 1.0;
  #endif
  #ifdef ARRCOMPENSATE
      Relong[q] = elongrate * min(baseElong+ (arr12Elong*ARR12[checkcell]/100.+arr1Elong*ARR1[checkcell]/100.),maxElong);
  #else
	  Relong[q]=elongrate*(baseElong + arr12Elong*ARR12[checkcell]/100.+arr1Elong*ARR1[checkcell]/100.) ;
  #endif
#endif
#ifdef LINEAREXPANSION
	  steptime=elongtime/(Hmax-((int)(2*HMZ)));
	  elongrate=1./steptime;
	  Relong[q]=elongrate;
#endif	  
	}

      first_time=1;
    }
#endif
  //#endif


  




  
  for(i=0;i<totcells;i++){

    JustChecked[i]=0;
    TempdivTF[i]=0;
    TempelongTF[i]=0;
  }


  double sharedivvalue = 0;
  double shareelongvalue = 0;
  //go through the meristem to determine growth critical parameters.
  int startcell=0;
  for (j = yLRCstraight; j<m;j++){
    if (Tc[disttoepi+1][j]==3){

      break;
    }
    int idist = disttoepi+1;
    if (Cell[idist][j] >-1 && Cell[idist][j] != startcell){
      startcell = Cell[idist][j];
      #ifdef epidermaldetermination
        sharedivvalue = divTF[  Cell[idist][j]  ];
        shareelongvalue = elongTF[  Cell[idist][j]  ];
      #endif
      #ifdef sidevascdetermination
        int disttosidevasc = disttoepi + totsizeepi+ totsizecort+ totsizeendo+ totsizeperi+1;
        sharedivvalue = divTF[  Cell[disttosidevasc][j]  ];
        shareelongvalue = elongTF[  Cell[disttosidevasc][j]  ];

      #endif
      #ifdef vascdetermination

        sharedivvalue = checkavevalue(j,1,divTF);

        shareelongvalue =checkavevalue(j,1,elongTF);
      #endif
      #ifdef averagedetermination
        sharedivvalue = checkavevalue(j,0,divTF);
        shareelongvalue =checkavevalue(j,0,elongTF);
      #endif 
      #ifdef lengthdetermination //define also a maxdivlength and maxelonglength
        if  (maxdivlength > j){
          sharedivvalue = divisionthreshold +10;
        }
        else {
          sharedivvalue = divisionthreshold -10;
        }
        if (maxelonglength > j){
          shareelongvalue = max(elongtreshold -10,0);
        }
        else{ 
          shareelongvalue = max(elongtreshold +10,0);
        }
      #endif

      int currentcell = 0;
      for (i = disttoepi+1;i<n-disttoepi-1;i++){
        if (currentcell != Cell[i][j] && Cell[i][j]>-1){
          currentcell = Cell[i][j];
          int altcell = Cell[n-i-1][j];
          TempdivTF[currentcell] = sharedivvalue;
          TempelongTF[currentcell] =shareelongvalue; 

        }
      }
    }
  }

  
  //give LRC celltypes. Not super relevant but helps makes sure every cell has the sharedvalues.
  startcell = 0;
  for (j = yLRCstraight; j<m;j++){
    i = disttolrc+2;
    if (Tc[i][j]==3){break;}
    if (Cell [i][j]!= -1 && Cell[i][j]!=startcell){ 
      startcell = Cell[i][j];
      int checkcell = Cell[disttoepi+1][CellMinJ[startcell]];
      if (checkcell<0)       checkcell = Cell[disttoepi+1][CellMinJ[startcell]+1];
      if( Tct[i][j]==1 && Tct[n-i-1][j]==1){
        TempdivTF[Cell[i][j]] = TempdivTF[checkcell];
        TempelongTF[Cell[i][j]]  =TempelongTF[checkcell];
        TempdivTF[Cell[n-i-1][j]] = TempdivTF[checkcell];
        TempelongTF[Cell[n-i-1][j]]  =TempelongTF[checkcell];
      }
    }
  }   
  
  double rdivaux,relong;
  //cell based Rgrow

  for(q=0;q<actcells;q++)
    {

  #ifdef EPIDERMALGROWTHCONTROL
    int checkcell = q;
    if (CellMinJ[q]>yLRCstraight){
      checkcell = epidermalcell(q);
    }
  #else
    int checkcell = q;
  #endif	
  #ifdef CORAUXCHECK
    int auxcheckcell = checkcell;
      if (CellMinJ[q]>yLRCstraight){
        auxcheckcell = cortexcell(q);
	  }
  #else
    int auxcheckcell = checkcell;
  #endif


      if(CellMaxJ[q]<yLRCstraight)//SCN: ignore growth
	Rgrow[q]=0.0000001;
      else if(CellMaxJ[q]<yLRCstraight)//inside curved part rootcap: ignore growth 
	Rgrow[q]=0.0000001;

    else if (TempelongTF[q]<elongtreshold){	 //first part of straight
      Rgrow[q]=Rdiv[q];
      int meani=(int)(0.5*(CellMinI[q]+CellMaxI[q]));
      int meanj=(int)(0.5*(CellMinJ[q]+CellMaxJ[q]));
      int celltype=Tct[meani][meanj];


		if (celltype < 5){Rgrow[q]*=1.1;}
    }
      
    else{// EZ and DZ expand and or differentiate
	  //EZ: expand and differentiate
	  //realize that if root cap is not dying
	  //cells alongside EZ will not be able to
	  //expand any further
	  if(CellHeight[q]<Hmax && Diff[q]<85)
	    {
	      Rgrow[q]=Relong[q];
         int meani=(int)(0.5*(CellMinI[q]+CellMaxI[q]));
         int meanj=(int)(0.5*(CellMinJ[q]+CellMaxJ[q]));
         int celltype=Tct[meani][meanj];
	    	if (celltype < 5){Rgrow[q]=Rdiv[q]*1.1;}
	      //Rgrow[q]=0.5*Relong[q]+0.1*Diff[q];//*(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+80*80));
	    }
	  else
	    Rgrow[q]=0.0000001;
	  }
    }
  int count = 0;
	

  for (int j=yLRCstraight;j < m;j++){
    for (int i=n/2-1;i>0;i--){
      int ii = n-i-1;
      if (Cell[i][j]!=-1){
        for (int boolint = 0; boolint<=1;boolint++){
          if (boolint==0){
            q = Cell[i][j]; //leftcell
          }
          else{
            q= Cell[ii][j]; //rightcell
          }

        if(JustChecked[q]==0 && CellZonationState[q]<200){
        #ifdef EPIDERMALGROWTHCONTROL
          int checkcell = q;
          if (CellMinJ[q]>yLRCstraight){
            checkcell = epidermalcell(q);
          }
        #else
          int checkcell = q;
        #endif
        #ifdef CORAUXCHECK
          int auxcheckcell = checkcell;
          if (CellMinJ[q]>yLRCstraight){
            auxcheckcell = cortexcell(q);
         }
        #else
          int auxcheckcell = checkcell;
        #endif



        H=CellHeight[q];
        int meani=(int)(0.5*(CellMinI[q]+CellMaxI[q]));
        int meanj=(int)(0.5*(CellMinJ[q]+CellMaxJ[q]));
        int celltype=Tct[meani][meanj];

        if(CellMaxJ[q]<yLRCstraight){//SCN: ignore growth
          CellZonationState[q]=0;
	       JustChecked[q]=1;
        }

          /*
         DETERMINATION OF DIVISION CELLS HERE
         */ 
        else if (TempdivTF[q]>divisionthreshold && CellZonationState[checkcell]<60){      

            CellZonationState[q]=30;
	        int Hdiv2=Hdiv;
            if (celltype <5&& CellMinJ[q]>((maxmzheight-yLRCstraight)/3+yLRCstraight) ){ // past 1/3 max MZ size lateral root cap cells don't get to divide
               Hdiv2=10000;
            }
      
        #ifndef LRCLENGTHCONTROL
	    if(celltype==1){//!!
	      Hdiv2=Hdiv2*1.5;
			cout << "got here anyway"<<std::endl;
        }
         #else
 
	      else if(celltype<5){//!!
	        Hdiv2=Hdiv2*1.5;
          }
       #endif


            JustChecked[q]=1;
            if(H>Hdiv2){	
              DivideCell(time,q);
              divided = true;
              if (boolint ==1){
                j=yLRCstraight;
                i = n/2-1;
                break;
              }
      		}
	         else if (H < Hmax){
              growcheck =growtiming(time,q,0);

            if (boolint ==1){
              j=yLRCstraight;
              i = n/2-1;
              break;
            }
            if (growcheck)grown = true;
	       }
	     }


      ///DECIDE growing as a non dividing cell
      else if (TempelongTF[q]<elongtreshold && CellZonationState[checkcell]<60){
         JustChecked[q]=1;

         CellZonationState[q]=50;
         growcheck =growtiming(time,q,0);
         if (boolint ==1){ 
           j=yLRCstraight;
           i = n/2-1;
           break;
         }

         if (growcheck)grown = true;
       }
		  
		  
		  
		///  DECIDE growing as an elongating or as a differentiated cell
	    else{//EZ or  DZ
	      JustChecked[q]=1;
	      //differentiation starts in EZ, proceeds in DZ

	      if((int)1000*Diff[q]==0)//only at start of diff
           Diff[q]+=0.001;
	      else
           Diff[q]+=tstep*speedup2*(diffrate-decaydiffrate*Diff[q]);


	      //solution Diff diff equation:
	      //Diff(t)=(diffrate/decaydiffrate)*(1-e^(-dt))+Diff(0)*e^(-dt)
	      //diffrate=0.0064; decaydiffrat0.000064
	      //how much time does reaching of DZ take: 
	      //required level to reach is 85 so
	      //85=(100)*(1-e^(-dt))+0.001*e^(-dt)
	      //can be approximated as
	      //85=100-100e^(-dt) (i.e leaving out 0.001*e^(-dt)
	      //0.85=1-e^(-dt) so -0.15=-e^(-dt) so 0.15=e^(-dt)
	      //ln(0.15)=-dt so
	      //t=-ln(0.15)/d=-ln(0.15)/0.000064=29642s=8.2 uur
	      //so if we want 7hr
	      //7uur=25200s we need d to be d=-ln(0.15)/25200s=0.000075


	      if(Diff[q]<85)//not into terminal differentiation
           {
     		  CellZonationState[q]=60;

		     if(H<Hmax)//still allowed to expand
		       {	

             Rgrow[q] = Relong[q];
             growcheck =growtiming(time,q,1);
             if (growcheck ){
             } 
             if (boolint ==1){
               j=yLRCstraight;
               i = n/2-1; 
               break;
             }
             if (growcheck)grown = true;
	        }
    		}
	      else if (CellZonationState[q]<=100){
		     CellZonationState[q]=100;     
         } 
         else if (CellZonationState[q]>100){
			  CellZonationState[q]=200;    
   	  }
	   }
	       }

        }
      }
    }
   }

  for (int q= 0; q<actcells;q++){
    if (CellMinJ[q]>yLRCstraight){
      if (ARR12[epidermalcell(q)]>ARRpinboundary){
        ARRpassed[q]=true;
      }
    }
  }
 
  #ifndef SMBMUTANT
    CullLateralRootcap(time);
  #endif
  if (divided) first_time = 0;
  maxmzsize(); 
  #define DYNAMICDP
  #ifdef DYNAMICDP
  Plasmodesmatapatterning();
  #endif

}

  

