/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "Arrays.hh"
#include "Auxin_params.hh"
#include "Basic_params.hh"
#include "Functions.hh"
#include "Param_config.hh"
#include "Plant_layout.hh"

void GrowCell(int time,int q,int growthtype)
{
  int i,j;
  int istart;
  int istop;
  int jstart;
  int jstop;
  double oldh,newh;
  double oldv,newv;
  double volratTF,volratnonTF;

   
  //shift all those cells lying above cell q, 
  //+ its above lying wall + its top membrane 
  //one row upward
  //do this for cell and directly nb wall

  //#define OLD
#define NEW  
#ifdef NEW  
  istart=CellMinI[q]-1;
  istop=CellMaxI[q]+1;
  jstart=CellMaxJ[q]-2;//top cell content (makes sure new cell grid point stops being membrane.
  jstop=m-2;

  i=CellMinI[q]+1;
  j=m-1;
  while(Tc[i][j]==3){
    j--;
  }
  jstop=j;
  for(j=jstop;j>=jstart;j--)//j>=jstart so including topmembrane cell q
    for(i=istart;i<=istop;i++)
      {
	//if it is left cell, if the point is on the right wall
	//and there is a cell right of that right wall: do not
	//do anything and leave it to the right cell
	//similarly if it is a right cell, if the point is on the left 
	//wall and there is a cell left of that left wall: do not
	//do anything and leave it to the left cell
	/////// CHANGED!!!//////


	if(  (CellMinI[q]<(n/2-1)) && i==istop &&(Tc[i+1][j+1]!=3  )   )  

	  {
	  }
	else if(CellMaxI[q]>(n/2-1) && i==istart && Tc[i-1][j+1]!=3 )

	  {
	  }
	else
	  {
	    //shift cell, membrane, wall locations
	    Tc[i][j+1]=Tc[i][j];
	    Tct[i][j+1]=Tct[i][j];
	    Tmt[i][j+1]=Tmt[i][j];
	    Cell[i][j+1]=Cell[i][j];
	    //shift Pin/Lax2/Aux1
	    PinPrePat[i][j+1]=PinPrePat[i][j];
	    Pin2PrePat[i][j+1]=Pin2PrePat[i][j];
	    Aux1PrePat[i][j+1]=Aux1PrePat[i][j];
	    
	    //shift auxin levels
	    Caux1[i][j+1]=Caux1[i][j];

	  }
      }
#endif  
  
  //now add a row to cell q and the directly nb wall 
  //and turn it (except for sides) into intracellular
  //instead of membrane space
  j=CellMaxJ[q]-1;
  double dilutionconversion =  (  (Hdiv/xstep)  - ((Hdiv/xstep)/2)  )/ (  (Hdiv/xstep)+1  - ((Hdiv/xstep)/2)  );
  oldh=(CellHeight[q]-Hdiv/xstep)*dilutionconversion +Hdiv/xstep ;
  //non-TF
  //grow from h=4 to h=9 so +5 but than add walls so -1, so effectively 4+4=8 so +4 so 4/5=0.8
  //grow from h=8 to h=17 so +9 but than add walls so -1, so effectively 8+8=16 so +8 so 8/9=0.888
  //newh=oldh+0.8; //was case with 2um per pixel, now with 1 um no longer the case.

  newh=oldh+dilutionconversion;//oldh+1;
  oldv=oldh*CellWidth[q];
  newv=newh*CellWidth[q];
  volratnonTF=(double)oldv/(double)newv;
  //TF
  //assume large part is not in cytoplasm and hence not diluted
  oldv=oldh*CellWidth[q];
  newv=newh*CellWidth[q];
  #ifndef DILUTIONFRACTION
  double fractcyto=1;//0.5;//!!!!!!!!!
  #endif
  volratTF=fractcyto*volratnonTF+(1.0-fractcyto);//=(fractcyto*((double)oldv/(double)newv)+(1-fractcyto));
  for(i=istart;i<=istop;i++)
    {
      //newly aroson gap is now filled with extra row by copying below row
      //so j=CellMaxJ[q]-1 was earlier one but topmost row cell q, just below membrane
      //and j+1 earlier was topmost row cell q, membrane, but is now changed into similar
      //state as j
      Tc[i][j+1]=Tc[i][j];
      Tct[i][j+1]=Tct[i][j];
      Tmt[i][j+1]=Tmt[i][j];
      Cell[i][j+1]=Cell[i][j];
      //correct Pin//Aux1 in extra lateral membrane?! (dilute??)
      PinPrePat[i][j+1]=PinPrePat[i][j];
      Pin2PrePat[i][j+1]=Pin2PrePat[i][j];
      Aux1PrePat[i][j+1]=Aux1PrePat[i][j]; 
      //nothing is done about auxin level so total auxin in
      //cell is not increasing?, so always diluted??
      //No, topmembrane cell q is shifted: takes along Caux1
      //lower part cell q is unchanged: keeps old Caux1
      //newly arosen gap is not overwritten: keesp old Caux1
      //so net increase of Caux1!
    }
  
  //DILUTE AUXIN!
  
  for(i=istart;i<=istop;i++)
    for(j=CellMinJ[q];j<=CellMaxJ[q]+1;j++)
      {
	Caux1[i][j]*=oldh/(oldh+1);
      }
  


  
  //implicit consideration of cytoplasmic or vacuolar growth
  //does not work when it is a substance undergoing diffusion
  //in that case, during vacuolar growth, the concentration
  //in the cytoplasm should stay the same, but so does the
  //amount of cytoplasm and hence the amount available for
  //diffusion, however by not explicitly considering cytoplasm
  //and vacuoles it appaers that concentration in cytoplasm
  //stays the same but volume increases and hence available
  //amount increases
  //auxin and PLTs diffuse
  //now PLTs do not dilute upon vacuolar growth, auxin does

  //dilute gene expression levels 
  if(growthtype==0)//cytoplasmic
    {

      //TF
      //ARF[q]*=volratTF;
      AA[q]*=volratTF;
      BB[q]*=volratTF;
      CC[q]*=volratTF;
      DD[q]*=volratTF;

      #ifndef PLTSNODILUTION 
      Plts[q]*=volratTF;//volratTF;//no dilution because it is all nuclear
      #endif
      ARR1[q]*=volratTF;
      ARR1RGA[q]*=volratTF;
      ARR12[q]*=volratTF;
      
      divTF[q]*=volratTF;
      elongTF[q]*=volratTF;

      //Diff[q]*=volratTF;
     

      //non-TF
#ifdef PINREGULATED
      //Pin[q]*=volratnonTF; /pins don't dilute in growth because their surface doesn't increase
      //Pin2[q]*=volratnonTF; /pins don't dilute in growth because their surface doesn't increase
#endif
#ifdef YUC3REGULATED
      Yuc3[q]*=volratnonTF;
      Gh317[q]*=volratnonTF;
      Gh33[q]*=volratnonTF;
#endif
#ifdef AUX1REGULATED
      Aux1[q]*=volratnonTF;
#endif
    }

  //else//vacuolar, no dilution of cytoplasmic conc
   
  
  TimePrevGrowth[q]=time;
  //in case of growth: do not renumber the cells
  //in this case lower left corner of cells may shift
  //so our method of mapping old to new cell ids 
  //does not work, as it assumes that lower left
  //corners of cells do not change, which is only
  //true during cell division (apart from new cell)
  //and cell culling (apart from removed cells)!!!!!
  //instead we just maintain old cell numbering
  //but make sure cells grow and cells above it shift
  //the recomputing off cellcorners and sizes does
  //work then
  FindCellCorners();
  FindCellSizes();

  CullOverflowCells(time); 


}


void DivideCell(int time,int q)
{

  int i,j;
  int divtime;
  int istart;
  int istop;
  int jstart;
  int jstop;
  int jwall;
  int oldid,newid,aboveid,oldmini,oldminj;
  //#define NOISYBUTSAMEINROW  

#ifdef NOISYBUTSAMEINROW  
  static int first_time=0;
  static int timememory;
  double R1,R2;
  int jump=53;
  int second;

  
  if(first_time==0)
    {
      timememory=-1;
      first_time=1;
    }

  //first time for this particular timestep that divide cell is called 
  if(time!=timememory)
    {
      for(j=yLRCstraight;j<m;j++)//fill array with new random numbers
	RandNr[j]=uniform();
    }

  //find random numbers for top and bottom daughter cell
  R1=RandNr[CellMinJ[q]];//random number for top daughter cell
  second=CellMinJ[q]+jump;
  if(second>=m-1)
    second=second-(m-1);
  R2=RandNr[second];//random number for bottom daughter cell
#endif


  //cell q is up for a division
  istart=CellMinI[q];
  istop=CellMaxI[q];
  jwall=CellHeight[q]/2;

  for(i=istart;i<=istop;i++)
    {
      //make wall:
      j=CellMinJ[q]+jwall;//wall
      Tc[i][j]=0;//wall
      Tct[i][j]=20;//wall
      Tmt[i][j]=0;//wall

      //make membrane above wall: bottom membrane top daughter cell
      j=CellMinJ[q]+jwall+1;//above wall
      Tc[i][j]=2;//membrane
      Tmt[i][j]=2;//bottom membrane 
      //copy protein pattern from bottom membrane mother cell
      PinPrePat[i][j]=PinPrePat[i][CellMinJ[q]];
      Pin2PrePat[i][j]=Pin2PrePat[i][CellMinJ[q]];
      Aux1PrePat[i][j]=Aux1PrePat[i][CellMinJ[q]];
      //make membrane below wall: top membrane lower daughter cell
      j=CellMinJ[q]+jwall-1;//below wall
      Tc[i][j]=2;//membrane
      Tmt[i][j]=1;//top membrane
      //copy protein pattern from top membrane mother cell
      PinPrePat[i][j]=PinPrePat[i][CellMaxJ[q]];
      Pin2PrePat[i][j]=Pin2PrePat[i][CellMaxJ[q]];
      Aux1PrePat[i][j]=Aux1PrePat[i][CellMaxJ[q]];
    }
 
  //Make sure to store old cell identities, locations,
  //and corresponding properties before doing division
  //clean the arrays
  int oldactcells=actcells;
  StoreOldSituation(oldactcells);
  //old identity dividing cell + location
  oldid=q;
  oldmini=CellMinI[q];
  oldminj=CellMinJ[q];
  //renumber cells after division
  NumberCells();
  FindCellCorners();
  FindCellSizes();
  //map old identities cells to their new ones
  MapOldToNewSituation(oldactcells);
  //find new identity dividing cell
  newid=Cell[oldmini][oldminj];
  //find identity top daughter cell
  aboveid=Cell[CellMinI[newid]][CellMaxJ[newid]+3];
  //make sure oldid maps to newid and aboveid


  //MAKE SURE NOT TO USE q ANY MORE:
  //q has divided into newid and aboveid
  //also possible renumbering has taken place



#ifndef RANDOMGROWTH
  double divtime1,divtime2;
  double elongtime1,elongtime2;
  double mu;
  double sigma;
  double sumdiv;
  double divrate;
  double sumelong;
  double elongrate;
  double steptime;
  
  divtime1=avgdivtime;//+(uniform()-0.5)*360;//divtime+(R1-0.5)*0.5*3600;
  divtime2=avgdivtime;//+(uniform()-0.5)*360;//divtime+(R2-0.5)*0.5*3600;



  
  if(Tct[CellMinI[newid]][CellMinJ[newid]]!=1)//non-lrc
    {
      //newid is bottom daughter cell
      //aboveid is top daughter cell

      
      //if second lowest one divides also asymmetric:
      //bottom one slow, top one normal
      int cellcounter=0;
      int cellid=newid;
      j=CellMinJ[newid];
      i=CellMinI[newid]+1;
      while(j>yLRCstraight)
	{
	  if(Cell[i][j]==cellid)//same cell
	    {
	      j--;
	    }
	  else if(Cell[i][j]==-1)//wall
	    {
	      j--;
	    }
	  else
	    {
	      cellcounter++;
	      cellid=Cell[i][j];
	      j--;
	    }
	}

      if(cellcounter==0)
	{
	  divtime2=1.35*divtime2;
	  divtime1=1.*divtime1;
	}
    }
 

 // #define DECORRELATE

#ifdef EXPONENTIALGROWTH
  sumdiv=0;
  for(i=HMZ;i<=2*HMZ;i++)
    sumdiv+=(1./((double)i));
#ifdef DECORRELATE
  double divtimeparent=sumdiv/Rdiv[newid];
  mu=divtimeparent;
  sigma=varperc*mu;
  divtime1=((gauss[(int)(R1*10000)]*sigma+mu));
  divtime2=((gauss[(int)(R2*10000)]*sigma+mu));
  //divtime1=divtimeparent+(R1-0.5)*0.5*3600;
  //divtime2=divtimeparent+(R2-0.5)*0.5*3600;
  double maxdivtime = 1.2;
  double mindivetime = 0.8;
  if(divtime1>1.5*avgdivtime)
    divtime1=1.5*avgdivtime;
  if(divtime1<0.5*avgdivtime)
    divtime1=0.5*avgdivtime;
  if(divtime2>1.5*avgdivtime)
    divtime2=1.5*avgdivtime;
  if(divtime2<0.5*avgdivtime)
    divtime2=0.5*avgdivtime;
#endif
  divrate=sumdiv/divtime1;
  Rdiv[aboveid]=divrate;
  divrate=sumdiv/divtime2;
  Rdiv[newid]=divrate;
#endif
#ifdef LINEARGROWTH
  steptime=divtime1/HMZ;
  divrate=1./steptime;
  Rdiv[aboveid]=divrate;
  steptime=divtime2/HMZ;
  divrate=1./steptime;
  Rdiv[newid]=divrate;
#endif
  elongtime1=avgelongtime;
  elongtime2=avgelongtime;
#ifdef EXPONENTIALEXPANSION
  sumelong=0;
  for(i=((int)HMZ);i<=Hmax;i++)
    sumelong+=(1./((double)i));
  elongrate=sumelong/elongtime1;
  Relong[aboveid]=elongrate;
  elongrate=sumelong/elongtime2;
  Relong[newid]=elongrate;
#endif
#ifdef LINEAREXPANSION
  steptime=elongtime1/(Hmax-((int)(2*HMZ)));
  elongrate=1./steptime;
  Relong[aboveid]=elongrate;
  steptime=elongtime2/(Hmax-((int)(2*HMZ)));
  elongrate=1./steptime;
  Relong[newid]=elongrate;
#endif	  
#endif
  TimePrevGrowth[aboveid]=time;
  TimePrevGrowth[newid]=time;
 
		       


  //if no noise you have to switch on this one!


  CellDivNr[aboveid]=CellDivNr[newid]+1;
  CellDivNr[newid]=CellDivNr[newid]+1;
  if(CellMinJ[newid]==(yLRCstraight+1))
    CellDivNr[newid]=0;
  
  CellTimeLastDiv[aboveid]=time;
  CellTimeLastDiv[newid]=time;
  Auxminpassed[aboveid]=Auxminpassed[newid];
  ARRpassed[aboveid]=ARRpassed[newid];
  ARF[aboveid]=ARF[newid];
  AA[aboveid]=AA[newid];
  BB[aboveid]=BB[newid];
  CC[aboveid]=CC[newid];
  DD[aboveid]=DD[newid];

  Plt[aboveid]=Plt[newid];
  Plts[aboveid]=Plts[newid];
  Pltexpr[aboveid]=Pltexpr[newid];
  Pltsexpr[aboveid]=Pltsexpr[newid];
  PP[aboveid]=PP[newid];
  Pin[aboveid]=Pin[newid];
  Pin2[aboveid]=Pin2[newid];
  Yuc3[aboveid]=Yuc3[newid];
  ARR1[aboveid]=ARR1[newid];
  ARR1RGA[aboveid]=ARR1RGA[newid];
  ARR12[aboveid]=ARR12[newid];



  Gh317[aboveid]=Gh317[newid];
  Gh33[aboveid]=Gh33[newid];
  Aux1[aboveid]=Aux1[newid];
  MeanCells[aboveid]=MeanCells[newid];
  Rgrow[aboveid]=Rgrow[newid];
  Rgrowavg[aboveid]=Rgrowavg[newid];
  JustChecked[aboveid]=JustChecked[newid];//!!
  CellZonationState[aboveid]=CellZonationState[newid];
  CellsInOZ[aboveid]=CellsInOZ[newid];
  Diff[aboveid]=Diff[newid];
  divTF[aboveid]=divTF[newid];
  elongTF[aboveid]=elongTF[newid];
  TempdivTF[aboveid]=TempdivTF[newid];
  TempelongTF[aboveid]=TempelongTF[newid];
  

  
#ifdef NOISYBUTSAMEINROW
  timememory=time;
#endif

}

void CullOverflowCells(int time)
{
  //due to the growth, cells get pushed over the edge
  //these cells need to be removed as soon as their
  //above cell wall drops over the edge to make sure
  //that only entire cells are part of the simulation
  //this automatically ensures that cell numbers do
  //not increase indefinetely

  int i,j,q;
  int ii,jj,iii,jjj;
  int counter;

  //scan line below topline
  //if still ok, it contains only external (3) or wall (0)
  //if culling is needed, it contains cytoplasm (1) or membrane (2) 
  //we do line below topline, as we need topline to create artificial
  //membrane/cell part for correct influx/efflux relation between
  //root and shoot


  int culling[totcells];
  for(q=0;q<totcells;q++)
    culling[q]=0;

  //first determine which cells need to be culled
  j=m-2;
  for(i=0;i<n;i++)
    {
      if(Tc[i][j]==1 || Tc[i][j]==2)//culling needed!
	{
	  //which cell is sitting here?
	  q=Cell[i][j];
	  culling[q]=1;
	}
    }
  counter=0;
  for(q=0;q<totcells;q++)
    {
      if(culling[q]==1)
	{
	  counter++;
	}
    }
  if(counter>0)//at least one cell culled
    {
      //now do the GRID based adjustments:
      //find where it sits in the field and remove it
      
      for(q=0;q<totcells;q++)
	{
	  if(culling[q]==1)
	    {
        //take epidermis length at both sides for root length calculation
			if (Tct[CellMinI[q]][CellMinJ[q]]==5){
           if (CellMinI[q]<(n/2)){
             leftsize+=CellHeight[q];
             totalrootcellnrL +=1;
           }
           else{
             rightsize+=CellHeight[q];
             totalrootcellnrR +=1;
           }

         }
	      //#define OLD
#define NEW
	      //always remove: 
	      //wall atop of to be removed cell 
	      //cell itself
	      //conditional remove:
	      //left and right walls
	      //only if no other cell there


	      for(ii=(CellMinI[q]-1);ii<=(CellMaxI[q]+1);ii++)
		for(jj=CellMinJ[q];jj<=(CellMaxJ[q]+1);jj++)
		  {

		    if(ii==(CellMinI[q]-1)  && 
		       (Cell[ii-1][jj]!=-1 || Cell[ii-1][jj-1]!=-1))
		      ;
		    else if(ii==(CellMaxI[q]+1) && 
			    (Cell[ii+1][jj]!=-1 || Cell[ii+1][jj-1]!=-1))
		      ;
		    else
		      {
			//put all to non-tissue status
			Tc[ii][jj]=3;
			Tct[ii][jj]=0;
			Tmt[ii][jj]=0;
			PinPrePat[ii][jj]=0;
			Pin2PrePat[ii][jj]=0;
			//more to come
			Cell[ii][jj]=-1;
			Caux1[ii][jj]=-10;//0;
         CellZonationState[Cell[ii][jj]]=0;
		      } 
		  }
	    }
	}

      //next do the CELL based readjustments:
      int oldactcells=actcells;
      StoreOldSituation(oldactcells);
      NumberCells();
      FindCellCorners();
      FindCellSizes();
      MapOldToNewSituation(oldactcells);
    }
}




void CullLateralRootcap(int time)
  {

  int i,j,k,ii,jj,iii,jjj;
  int q,qq,minqq;
  int meanJ1;
  int meanJ2;
  FILE *f;
  char s[1000];
  sprintf(s,"%s/Lateralrootcapculling",dirname);
  int minJ = 0;

  f=fopen(s,"a");
  for(k=0;k<2;k++)
    {
    if(k==0)
	  {
	  //check left lateral root cap:
	  i=disttolrc+1;
      }
    else
	{
	  //check right lateral root cap
	  i=n-1-(disttolrc+1);
	}
    for(j=yLRCstraight;j<=m-1;j++)
	  {

     if (Tc[i][j]==3){ //no longer any lateral root cap
       break;
     }
	  if(Tc[i][j]==2)//uppermost lateral root cap found
	    {
	    q=Cell[i][j];
	    if (minJ==0)
	      {
		  minJ = CellMinJ[q];
			
		  }
      
#ifdef PLTDEPZONATION
	    //find neighbouring epidermal cell
	    meanJ1=int(0.5*(CellMinJ[q]+CellMaxJ[q]));
	    meanJ2= meanJ1+1;
       int cellbelow = Cell[CellMinI[q]+1][CellMinJ[q]-3];
       int meanJ3 = int(0.5*(CellMinJ[cellbelow]+CellMaxJ[cellbelow]));
       int meanJ4 = meanJ3+1;
	    ii=i;
	    if(k==0)//left root cap so epidermis to the right
		  {
		  bool failed = true;
		  while(failed && ii < n-2){
		    if (Cell[ii][meanJ1]!=Cell[i][meanJ1] && Cell[ii][meanJ1] != -1){
		      qq=  Cell[ii][meanJ1];
 		      failed = false;
            
            if (Cell[ii][meanJ3] != -1){
             minqq = Cell[ii][meanJ3];

            }
            else if (CellMinJ[q]>yLRCstraight+1){ 
              minqq = Cell[ii][meanJ4];
              }
            else{
              minqq = Cell[ii][CellMinJ[q]+1];
	         }
            
			 }
			 else if (failed && Cell[ii][meanJ2]!=Cell[i][meanJ2] && Cell[ii][meanJ2] != -1){
		      qq=  Cell[ii][meanJ2];
		      failed = false;
           if (Cell[ii][meanJ3] != -1){
             minqq = Cell[ii][meanJ3];

            }
            else if (CellMinJ[q]>yLRCstraight+1){ 
              minqq = Cell[ii][meanJ4];
              }
            else{
              minqq = Cell[ii][CellMinJ[q]+1];
	         }
			  }
			ii++;
		    }
		  }
	    else if (k==1)//right root cap so epidermis to the left
		  {
		  bool failed = true;
		  while(failed&& ii >0){
		    if (Cell[ii][meanJ1]!=Cell[i][meanJ1] && Cell[ii][meanJ1] != -1){
		      qq=  Cell[ii][meanJ1];
			   failed = false;
           
             if (Cell[ii][meanJ3] != -1){
             minqq = Cell[ii][meanJ3];

            }
            else if (CellMinJ[q]>yLRCstraight+1){ 
              minqq = Cell[ii][meanJ4];
              }
            else{
              minqq = Cell[ii][CellMinJ[q]+1];
	         }
			  }
			else if (failed && Cell[ii][meanJ2]!=Cell[i][meanJ2] && Cell[ii][meanJ2] != -1){
			  qq=  Cell[ii][meanJ2];
			  failed=false;
           if (Cell[ii][meanJ3] != -1){
             minqq = Cell[ii][meanJ3];
            }
            else if (CellMinJ[q]>yLRCstraight+1){ 
              minqq = Cell[ii][meanJ4];
              }
            else{
              minqq = Cell[ii][CellMinJ[q]+1];
	         }
			  }
			ii--;
		   }
		 }
      #ifndef LATCRITERIUM // if you define this, whatever the elongation zone determinant is determines where the lateral root cap ends.
        if  (  (qq==-1)||( (CellHeight[qq]>(HMZ*4+1))&&   (CellMinJ[q]>yLRCstraight) )  )
      #else
        #ifdef NOSIZELATCRITERIUM

         if(  (qq==-1)|| ( CellZonationState[qq]>=60 && CellZonationState[minqq]>=50 )&&   (CellMinJ[q]>yLRCstraight)  ) //no longer MZ bounded.
        #else
          if (  (qq==-1)|| (   ( CellHeight[qq]>(HMZ*4+1)&&(CellZonationState[qq]>=60 && CellZonationState[minqq]>=50) )&&   (CellMinJ[q]>yLRCstraight) ) ) //no longer MZ bounded.
        #endif
      #endif

#else
	    if(CellMinJ[q]>yLRC1)//PERHAPS THIS THAN SHOULD BE yMZEZ, ONCE ADJUSTED
#endif  
		  {
        // WE HAVE FOUND A CELL TO REMOVE!
		  fprintf(f,"%f 1\n",((double)time/3600.));
          //removing cell itself+ wall above
		  for(ii=CellMinI[q];ii<=CellMaxI[q];ii++){
		    for(jj=CellMinJ[q];jj<=m-1;jj++)//(CellMaxJ[q]+1)
		      {    
		      //put all to non-tissue status
			  Tc[ii][jj]=3;
			  Tct[ii][jj]=0;
			  Tmt[ii][jj]=0;
			  PinPrePat[ii][jj]=0;
			  Pin2PrePat[ii][jj]=0;
			  //more to come
			  Cell[ii][jj]=-1;
			  Caux1[ii][jj]=-10;//0;	

		      }
		    }

		  if(k==0)
		    {
		    //wall left can go for left lrc
		    iii=CellMinI[q]-1;
		    for(jjj=CellMinJ[q];jjj<=m-1;jjj++)//(CellMaxJ[q]+1)
			  {
			  Tc[iii][jjj]=3;
			  Tct[iii][jjj]=0;
			  Tmt[iii][jjj]=0;

			  Cell[iii][jjj]=-1;
			  Caux1[iii][jjj]=-10;//0;
			  }
		    }
		  else if (k==1) 
		    {
		    //wall right can go for right lrc
		    iii=CellMaxI[q]+1;
		    for(jjj=CellMinJ[q];jjj<=m-1;jjj++)//(CellMaxJ[q]+1)
			  {
			  Tc[iii][jjj]=3;
			  Tct[iii][jjj]=0;
			  Tmt[iii][jjj]=0;
			  Cell[iii][jjj]=-1;
			  Caux1[iii][jjj]=-10;//0;
			  }
		    }
		  int oldactcells=actcells;
		  StoreOldSituation(oldactcells);
		  NumberCells();
		  FindCellCorners();
		  FindCellSizes();
		  MapOldToNewSituation(oldactcells);
        break;
		  }
	    }
	  }
    }
    fclose(f);
  }



void StoreOldSituation(int oldactcells)
{
  int i,k;

  //Make sure to store old cell identities, locations,
  //and corresponding properties before doing division
  //clean the arrays
  for(i=0;i<totcells;i++)
    {
      OldSit[i][0]=-1;
      OldSit[i][1]=-1;
      OldSit[i][2]=-1;
      
      OldTimePrevGrowth[i]=-1;
      OldAuxminpassed[i]=-1;
      OldARRpassed[i]=-1;
      OldARF[i]=-1;
      OldAA[i]=-1;
      OldBB[i]=-1;
      OldCC[i]=-1;
      OldDD[i]=-1;
      OldPlt[i]=-1;
      OldPlts[i]=-1;
      OldPltsexpr[i]=-1;
      OldPP[i]=-1;
      OldPin[i]=-1;
      OldPin2[i]=-1;
      OldYuc3[i]=-1;
      OldARR1[i]=-1;
      OldARR1RGA[i]=-1;
      OldARR12[i]=-1;
      OldGh317[i]=-1;
      OldGh33[i]=-1;
      OldAux1[i]=-1;
      OldMeanCells[i]=-1;
      OldRgrow[i]=-1;
      OldRgrowavg[i]=-1;
      OldRdiv[i]=-1;
      OldRelong[i]=-1;
      OldCellZonationState[i]=-1;
      OldCellDivNr[i]=-1;
      OldCellTimeLastDiv[i]=-1;
      OldCellsInOZ[i]=-1;
      OldDiff[i]=-1;
      OlddivTF[i]=-1;
      OldelongTF[i]=-1;
      TempOlddivTF[i]=-1;
      TempOldelongTF[i]=-1;    
      OldJustChecked[i]=-1;
    }
  //fill them
  for(i=0;i<oldactcells;i++)
    {
      OldSit[i][0]=i;//old cell nr
      OldSit[i][1]=CellMinI[i];//min i value
      OldSit[i][2]=CellMinJ[i];//min j value
      
      OldTimePrevGrowth[i]=TimePrevGrowth[i];
      OldAuxminpassed[i] = Auxminpassed[i];
      OldARRpassed[i] = ARRpassed[i];
      OldARF[i]=ARF[i];
      OldAA[i]=AA[i];
      OldBB[i]=BB[i];
      OldCC[i]=CC[i];
      OldDD[i]=DD[i];
      OldPlt[i]=Plt[i];
      OldPlts[i]=Plts[i];
      OldPltsexpr[i]=Pltsexpr[i];
      OldPP[i]=PP[i];
      OldPin[i]=Pin[i];
      OldPin2[i]=Pin2[i];
      OldYuc3[i]=Yuc3[i];
      OldARR1[i]=ARR1[i];
      OldARR1RGA[i]=ARR1RGA[i];
      OldARR12[i]=ARR12[i];
      OldGh317[i]=Gh317[i];
      OldGh33[i]=Gh33[i];
      OldAux1[i]=Aux1[i];
      OldMeanCells[i]=MeanCells[i];
      OldRgrow[i]=Rgrow[i];
      OldRgrowavg[i]=Rgrowavg[i];
      OldRdiv[i]=Rdiv[i];
      OldRelong[i]=Relong[i];
      OldCellZonationState[i]=CellZonationState[i];
      OldCellDivNr[i]=CellDivNr[i];
      OldCellTimeLastDiv[i]=CellTimeLastDiv[i];
      OldCellsInOZ[i]=CellsInOZ[i];
      OldDiff[i]=Diff[i];
      OlddivTF[i]=divTF[i];
      OldelongTF[i]=elongTF[i];
      TempOlddivTF[i]=TempdivTF[i];
      TempOldelongTF[i]=TempelongTF[i];
      OldJustChecked[i]=JustChecked[i];
    } 
}


void TrackCells(int time)
{
  static int first_time=0;
  static int peri_i=disttoepi+totsizeepi+totsizeendo+totsizecort+1;
  static int peri_j=yLRC1-5*HMZ-5-1;
  int i;
  FILE *f;
  char fname[600];

  if(first_time==0)
    {
      memid=-1;
      for(i=0;i<3000;i++)
	    idlist[i]=-1;

      first_time=1;
      idcounter=0;
    }
  else
    {
      if(Cell[peri_i][peri_j]!=-1 && Cell[peri_i][peri_j]!=memid)
	{
	  memid=Cell[peri_i][peri_j];
	  idcounter++;
	  idlist[idcounter]=memid;
	  //printf("new id %i idcounter is %i at time %i\n",memid,idcounter,time);
	  
	}
      if(idcounter>=3000)
	printf("idcounter is groter dan 3000\n");

      
      //printf("id first found cell is %i\n",idlist[1]);

      if(time%100==0 && idcounter>0)
	{
	  for(i=1;i<=min(idcounter,2999);i++)
	    {
	      sprintf(fname,"%s/TrackCell_%d",dirname,i);
	      f=fopen(fname,"a");
	      fprintf(f,"%i\t%i\t%f\t%f\t%i\t%f\t%i\t%i\n",time,idlist[i],MeanCells[idlist[i]],PP[idlist[i]],CellHeight[idlist[i]],(0.5*(CellMinJ[idlist[i]]+CellMaxJ[idlist[i]])),CellDivNr[idlist[i]],CellTimeLastDiv[idlist[i]]);
	      fclose(f);
	    }
	}
    }
}



void MapOldToNewSituation(int oldactcells)
{
  int i,j,k;

  //map old identities cells to their new ones
  for(i=0;i<actcells;i++)//new cells
    for(j=0;j<oldactcells;j++)//old cells
      {
	//find if old cell id j corresponds to new cell id i
	if(OldSit[j][1]==CellMinI[i] && OldSit[j][2]==CellMinJ[i])
	  MapCellNrs[i]=j;//new cell i was old cell j
      }


  //check whether for cells we are tracking ids changed
  //if so, adjust

  int renummemid=-1;
  int renumidlist[3000];
  for(i=0;i<3000;i++)
    renumidlist[i]=-1;

  for(i=0;i<actcells;i++)
    {
      j=MapCellNrs[i];

      if(memid==j)
	{
	  renummemid=i;
	}

      for(k=1;k<=idcounter;k++)
	{
	  if(idlist[k]==j)
	    {
	      renumidlist[k]=i;
	    }
	}
    }
  memid=renummemid;
  for(i=0;i<3000;i++)
    idlist[i]=renumidlist[i];


   for(i=0;i<actcells;i++)
     {
       TimePrevGrowth[i]=OldTimePrevGrowth[MapCellNrs[i]];
       Auxminpassed[i]=OldAuxminpassed[MapCellNrs[i]];
       ARRpassed[i]=OldARRpassed[MapCellNrs[i]];
       ARF[i]=OldARF[MapCellNrs[i]];
       AA[i]=OldAA[MapCellNrs[i]];
       BB[i]=OldBB[MapCellNrs[i]];
       CC[i]=OldCC[MapCellNrs[i]];
       DD[i]=OldDD[MapCellNrs[i]];
       Plt[i]=OldPlt[MapCellNrs[i]];
       Plts[i]=OldPlts[MapCellNrs[i]];
       Pltsexpr[i]=OldPltsexpr[MapCellNrs[i]];
       PP[i]=OldPP[MapCellNrs[i]];
       Pin[i]=OldPin[MapCellNrs[i]];
       Pin2[i]=OldPin2[MapCellNrs[i]];
       Yuc3[i]=OldYuc3[MapCellNrs[i]];
       ARR1[i]=OldARR1[MapCellNrs[i]];
       ARR1RGA[i]=OldARR1RGA[MapCellNrs[i]];
       ARR12[i]=OldARR12[MapCellNrs[i]];
       Gh317[i]=OldGh317[MapCellNrs[i]];
       Gh33[i]=OldGh33[MapCellNrs[i]];
       Aux1[i]=OldAux1[MapCellNrs[i]];
       MeanCells[i]=OldMeanCells[MapCellNrs[i]];
       Rgrow[i]=OldRgrow[MapCellNrs[i]];
       Rgrowavg[i]=OldRgrowavg[MapCellNrs[i]];
       Rdiv[i]=OldRdiv[MapCellNrs[i]];
       Relong[i]=OldRelong[MapCellNrs[i]];
       CellZonationState[i]=OldCellZonationState[MapCellNrs[i]];  
       CellDivNr[i]=OldCellDivNr[MapCellNrs[i]];  
       CellTimeLastDiv[i]=OldCellTimeLastDiv[MapCellNrs[i]];  
       CellsInOZ[i]=OldCellsInOZ[MapCellNrs[i]];
       Diff[i]=OldDiff[MapCellNrs[i]];
       divTF[i]=OlddivTF[MapCellNrs[i]];
       elongTF[i]=OldelongTF[MapCellNrs[i]];
       TempdivTF[i]=TempOlddivTF[MapCellNrs[i]];
       TempelongTF[i]=TempOldelongTF[MapCellNrs[i]];
       JustChecked[i]=OldJustChecked[MapCellNrs[i]];
     }
}

