/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "Arrays.hh"
#include "Auxin_params.hh"
#include "Basic_params.hh"
#include "Functions.hh"
#include "Param_config.hh"
#include "Plant_layout.hh"


void InitConditions()
{
  int i,j,k,q;

  Plasmodesmatapatterning();

  #ifndef LOWINITIALPLT  //Low initial PLT settings turned on in Config_params.hh
  //set gene expression levels:
  for(q=0;q<actcells;q++)
    {
      MeanCells[q]=100;
      ARF[q]=00;
      AA[q]=20;
      BB[q]=20;
      CC[q]=30;
      DD[q]=55;
      Plt[q]=0;
      Plts[q]=0;
      ARR1[q]=0;
      ARR12[q]=0; 
      Aux1[q]=100;
      Pin[q]=100;
      Pin2[q]=100;
      PP[q]=0;
      divTF[q]=100;
      if(CellMaxJ[q]<yLRC1){
        Plts[q]=125; 
        if (Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]==12){
	       divTF[q] = 100;
          Plts[q]=125;
          ARF[q]=40;
          AA[q]=20;
          BB[q]=20;
          CC[q]=30;
          DD[q]=55;

        }
        else if (Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]>=8 &&
        Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]<=11){
           ARF[q]=00;
           AA[q]=20;
           BB[q]=20;
           CC[q]=30;
           DD[q]=55;
           Plts[q]=125;

        }
        else if (  Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]==5 ){
          ARF[q]=00;
          AA[q]=20;
          BB[q]=20;
          CC[q]=30;
          DD[q]=55;
          Plts[q]=125;
          }
        else if (  Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]<5 &&Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]>=1  ){
          ARF[q]=00;
          AA[q]=20;
          BB[q]=20;
          CC[q]=30;
          DD[q]=55;
          Plts[q]=125;
          }
        else{ //if we are below HJC but in particular tissues
          ARF[q]=00;
          AA[q]=20;
          BB[q]=20;
          CC[q]=20;
          DD[q]=40;
          Plts[q]=75;
          }



        Yuc3[q]=00;
        Gh33[q]=100;
        Gh317[q]=00;
      }
     else{
       Plts[q]=250;
      }

      TimePrevGrowth[q]=0;
      Diff[q]=0;
      CellZonationState[q]=0;
    }
  
  #else

//set gene expression levels:
  for(q=0;q<actcells;q++)
    {
      MeanCells[q]=100;
      ARF[q]=00;
      AA[q]=20;
      BB[q]=20;
      CC[q]=30;
      DD[q]=30;
      Plt[q]=0;
      Plts[q]=0;
      ARR1[q]=0;
      ARR12[q]=0; 
      Aux1[q]=100;
      Pin[q]=100;
      Pin2[q]=100;
      PP[q]=0;
      divTF[q]=100;
      if(CellMaxJ[q]<yLRC1){
        Plts[q]=65; 
        if (Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]==12){
	       divTF[q] = 100;
          Plts[q]=65;
          ARF[q]=40;
          AA[q]=20;
          BB[q]=20;
          CC[q]=30;
          DD[q]=20;

        }
        else if (Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]>=8 &&
        Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]<=11){
           ARF[q]=00;
           AA[q]=20;
           BB[q]=20;
           CC[q]=30;
           DD[q]=20;
           Plts[q]=50;

        }
        else if (  Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]==5 ){
          ARF[q]=00;
          AA[q]=20;
          BB[q]=20;
          CC[q]=30;
          DD[q]=20;
          Plts[q]=50;
          }
        else if (  Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]<5 &&Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2]>=1  ){
          ARF[q]=00;
          AA[q]=20;
          BB[q]=20;
          CC[q]=30;
          DD[q]=20;
          Plts[q]=50;
          }
        else{ //if we are below HJC but in particular tissues
          ARF[q]=00;
          AA[q]=20;
          BB[q]=20;
          CC[q]=20;
          DD[q]=20;
          Plts[q]=50;
          }



        Yuc3[q]=00;
        Gh33[q]=100;
        Gh317[q]=00;
      }
     else{
       Plts[q]=50;
      }

      TimePrevGrowth[q]=0;
      Diff[q]=0;
      CellZonationState[q]=0;
    }
  #endif
 
	  
  //set auxin concentration levels:
  for(i=0;i<n;i++){
    for(j=0;j<m;j++){
	  if(Tc[i][j]!=3 && Cell[i][j]!=-1){
	    if(j<10)
	      Caux1[i][j]=50;
	    else if(j<30)
	      Caux1[i][j]=50;
	    else if(j<60)
	     Caux1[i][j]=50;
	    else if(j<100)
	     Caux1[i][j]=50;
	    else if(j<250)
	     Caux1[i][j]=50;
	    else if(j<500)
	      Caux1[i][j]=50;
	    else
	      Caux1[i][j]=50;  
	  }
	}
  }
}
