/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "Arrays.hh"
#include "Auxin_params.hh"
#include "Basic_params.hh"
#include "Functions.hh"
#include "Param_config.hh"
#include "Plant_layout.hh"


void Plasmodesmatapatterning(){
	
  //in Tct[n][m] 
  //0 = no cell
  //1 = lateral root cap 1
  //2 = lateral root cap 2
  //3 = lateral root cap 3
  //4 = lateral root cap 4
  //5 = epi
  //6 = cort
  //7 = endo
  //8 = peri
  //9 = vasc 1
  //10 = vasc 2
  //11 = vasc 3
  //12 = QC
  //13 = columella 1 (lowest up)
  //14 = columella 2
  //15 = columella 3
  //16 = columella 4
  //17 = columella 5 (highest up), connected to epidermis
  //18 = columella initials
  for (int q = 0; q<actcells;q++){
  
      //tmt     plasmaarray
    //1 up   -- 0
    //2 down -- 1
    //3 in   -- 2
    //4 out  -- 3
  
  bool dynamicpd = true;
  for (int dir = 0; dir<4;dir++){
    Plasmodesmatapattern[q][dir]=1;
  }

  if (dynamicpd){
    int currentcelltype = Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMaxJ[q]+CellMinJ[q])/2];
      if (CellMaxJ[q]<= yLRCstraight-2){ //everything but the top layer of cells below YLRCstraight
	    if (currentcelltype == 12){
          Plasmodesmatapattern[q][0]=3;
          Plasmodesmatapattern[q][1]=3;
          Plasmodesmatapattern[q][2]=3;
          Plasmodesmatapattern[q][3]=3;
	    }
	    else if(currentcelltype ==17 ){
         if(  ( (CellMaxI[q]< (n/2))  && (Tct[CellMinI[q]-3][(CellMaxJ[q]+CellMinJ[q])/2]==17) )||
              ( (CellMinI[q]>= (n/2)) && (Tct[CellMaxI[q]+3][(CellMaxJ[q]+CellMinJ[q])/2]==17) )  )  {         
           Plasmodesmatapattern[q][0]=3;
           Plasmodesmatapattern[q][1]=3; 
           Plasmodesmatapattern[q][2]=3; 
           Plasmodesmatapattern[q][3]=6; 
         }
         //else if (Tct[(CellMinI[q]+CellMaxI[q])/2][(CellMinJ[q]+CellMaxJ[q])/2]==5){
         if(   ( (CellMaxI[q]< (n/2) )  && (Tct[CellMinI[q]-3][(CellMaxJ[q]+CellMinJ[q])/2]==5) ) ||
               ( (CellMinI[q]>= (n/2) ) && (Tct[CellMaxI[q]+3][(CellMaxJ[q]+CellMinJ[q])/2]==5) )   ){ 
           Plasmodesmatapattern[q][0]=6; //down in root becomes left in model
           Plasmodesmatapattern[q][1]=3; // up becomes right
           Plasmodesmatapattern[q][2]=3; // left becomes up
           Plasmodesmatapattern[q][3]=6; // right becomes down
         }
       }
       else if (currentcelltype ==16 ){
         Plasmodesmatapattern[q][0]=1;  
         Plasmodesmatapattern[q][1]=1;
         Plasmodesmatapattern[q][2]=6;
         Plasmodesmatapattern[q][3]=1;
       }
       else if (currentcelltype ==5){
         if(   ( (CellMaxI[q]< (n/2) )  && (Tct[CellMaxI[q]+3][(CellMaxJ[q]+CellMinJ[q])/2]==17) ) ||
               ( (CellMinI[q]>= (n/2) ) && (Tct[CellMinI[q]-3][(CellMaxJ[q]+CellMinJ[q])/2]==17) )   ){ 

             Plasmodesmatapattern[q][0]=10;
             Plasmodesmatapattern[q][1]=6; 
             Plasmodesmatapattern[q][2]=1; 
             Plasmodesmatapattern[q][3]=1; 
         }
         else{

             Plasmodesmatapattern[q][0]=10;
             Plasmodesmatapattern[q][1]=10; 
             Plasmodesmatapattern[q][2]=1; 
             Plasmodesmatapattern[q][3]=1; 
          }        
       }
	    else if (currentcelltype == 6){
         if (Tct[(CellMaxI[q]+CellMinI[q])/2][(CellMinJ[q])-4]==17){ //if the cell below you is columella upper, you are an initial cell
		      Plasmodesmatapattern[q][0]=6;
            Plasmodesmatapattern[q][1]=3;
            Plasmodesmatapattern[q][2]=3;
            Plasmodesmatapattern[q][3]=3;
		   }
		   else if (  ( (CellMaxI[q]< (n/2)) &&  (CellMinI[q]> disttoepi+totsizeepi+totsizecort) )||
                     ( (CellMinI[q]>= (n/2)) && (CellMaxI[q]< n-(disttoepi+totsizeepi+totsizecort)) )  ){ //the cell file connecting to the initial cells
           Plasmodesmatapattern[q][0]=10;
		     Plasmodesmatapattern[q][1]=6;
		     Plasmodesmatapattern[q][2]=3;
           Plasmodesmatapattern[q][3]=1;
	      }
	      else{
			  Plasmodesmatapattern[q][0]=10;
		     Plasmodesmatapattern[q][1]=10;
		  	  Plasmodesmatapattern[q][2]=3;
           Plasmodesmatapattern[q][3]=1;
	      }
 		 }

	    else if (currentcelltype == 7){
         if (Tct[(CellMaxI[q]+CellMinI[q])/2][CellMinJ[q]-4]==17){ //if the cell below you is columella upper, you are an initial cell
		     Plasmodesmatapattern[q][0]=6;
           Plasmodesmatapattern[q][1]=3;
           Plasmodesmatapattern[q][2]=3;
           Plasmodesmatapattern[q][3]=3;
		   }
		   else{
		     Plasmodesmatapattern[q][0]=10;
			  Plasmodesmatapattern[q][1]=6;
			  Plasmodesmatapattern[q][2]=3;
           Plasmodesmatapattern[q][3]=3;
		   }
	    }
 
      }
      else if (CellMinJ[q] < yLRCstraight ){ //catches the top row of all cells that are not dividing
	    if (currentcelltype <5){
            Plasmodesmatapattern[q][0]=10;
            Plasmodesmatapattern[q][1]=1; 
            Plasmodesmatapattern[q][2]=1;
            Plasmodesmatapattern[q][3]=1;
	      }
	    else if (currentcelltype == 5){
            Plasmodesmatapattern[q][0]=10;
            Plasmodesmatapattern[q][1]=10; 

            Plasmodesmatapattern[q][2]=1;
            Plasmodesmatapattern[q][3]=1;
	      }
	    else if (currentcelltype == 6){
         Plasmodesmatapattern[q][0]=10;
         Plasmodesmatapattern[q][1]=10;
         Plasmodesmatapattern[q][2]=3;
         Plasmodesmatapattern[q][3]=1;
	    }	
	    else if (currentcelltype == 7){
         Plasmodesmatapattern[q][0]=10;
         Plasmodesmatapattern[q][1]=10;
         Plasmodesmatapattern[q][2]=3;
         Plasmodesmatapattern[q][3]=3;
       }
	    else if (currentcelltype == 8){

          Plasmodesmatapattern[q][0]=10;
          Plasmodesmatapattern[q][1]=3;
          Plasmodesmatapattern[q][2]=3; 
          Plasmodesmatapattern[q][3]=3;

	      }		    
	    else if (currentcelltype == 9|| currentcelltype == 10 || currentcelltype == 11){
          Plasmodesmatapattern[q][0]=10;
          Plasmodesmatapattern[q][1]=3;
	      Plasmodesmatapattern[q][2]=3;
          Plasmodesmatapattern[q][3]=3;
	   
	      }		  
        }

      else{ //catches all the regular growing cells

	    if (currentcelltype <5){
            Plasmodesmatapattern[q][0]=10;
            Plasmodesmatapattern[q][1]=10; 
            Plasmodesmatapattern[q][2]=1;
            Plasmodesmatapattern[q][3]=1;
	      }
        else if ((Tct[CellMaxI[q]-1][CellMaxJ[q]-1]==5)){ //epidermis

          if( CellMinJ[q]-3 <yLRCstraight  ) {
            Plasmodesmatapattern[q][0]=10;
            Plasmodesmatapattern[q][1]=10; //down 20 for compensation lack of movement 
            Plasmodesmatapattern[q][2]=1;
            Plasmodesmatapattern[q][3]=1;
            }
          else{
            Plasmodesmatapattern[q][0]=10;
            Plasmodesmatapattern[q][1]=10; 
            Plasmodesmatapattern[q][2]=1;
            Plasmodesmatapattern[q][3]=1;
         }
       }

	    else if ((Tct[CellMaxI[q]-1][CellMaxJ[q]-1]==6)){//cortex

          Plasmodesmatapattern[q][0]=10;
          Plasmodesmatapattern[q][1]=10;
	      Plasmodesmatapattern[q][2]=3;
          Plasmodesmatapattern[q][3]=1;
	    } 
	    else if ((Tct[CellMaxI[q]-1][CellMaxJ[q]-1]==7)){//endodermis
		  Plasmodesmatapattern[q][0]=10;
          Plasmodesmatapattern[q][1]=10;
	      Plasmodesmatapattern[q][2]=3;
          Plasmodesmatapattern[q][3]=3;
	    }
	    else if ((Tct[CellMaxI[q]-1][CellMaxJ[q]-1]==8)){//pericycle
		  Plasmodesmatapattern[q][0]=10;
          if (CellMinJ[q]-5<yLRCstraight){
            Plasmodesmatapattern[q][1]=10;
          }
          else{
            Plasmodesmatapattern[q][1]=10;
          }
          Plasmodesmatapattern[q][2]=1;
          Plasmodesmatapattern[q][3]=3;
        }
        else if ((Tct[CellMaxI[q]-1][CellMaxJ[q]-1]>=9 &&Tct[CellMaxI[q]-1][CellMaxJ[q]-1]<=11 )){//vascilature
		    Plasmodesmatapattern[q][0]=10;
          if (CellMinJ[q]-5<yLRCstraight){
            Plasmodesmatapattern[q][1]=10;
          }
          else{
            Plasmodesmatapattern[q][1]=10;
          }
	      Plasmodesmatapattern[q][2]=1;
          Plasmodesmatapattern[q][3]=1;
	    }
	  }
      #ifdef CLOSEROOT
		if ( CellZonationState[q]==200 && CellZonationState[ Cell[CellMaxI[q]-1][CellMinJ[q]-3] ] <200 ){// the lowest non-root tissue
        Plasmodesmatapattern[q][1]=0; //we close connection with the downwards part where non root connects to the root
      }
      else if  ( CellZonationState[q]<200 && CellZonationState[Cell[CellMaxI[q]-1][CellMaxJ[q]+3] ] ==200){ //the highest root which is connected to non-root
		  Plasmodesmatapattern[q][0]=0; //we close connetion with the upward part where root connects to nonroot
      }
      #endif
    }
  }
}
