/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "Arrays.hh"
#include "Auxin_params.hh"
#include "Basic_params.hh"
#include "Functions.hh"
#include "Param_config.hh"
#include "Plant_layout.hh"


void ExtraMeasurements(int t)
{
  FILE *f;
  char s[800];
  int toplrc;
  int sizetoplrc;
  double auxlrc;
  int pospinflip;
  int i,j;

  //top position lateral root cap
  //size topmost lateral root cap cell
  //auxin level topmost lateral root cap cell
  toplrc=-1;
  sizetoplrc=-1;
  i=disttolrc+1;
  j=m-1;
  while(Tc[i][j]==3)
    j--;
  if(!(Tct[i][j-1]==1))
    {
      printf("something is wrong in finding top lrc\n");
      exit(1);
    }
  else
    {
      toplrc=j-1;
      sizetoplrc=CellHeight[Cell[i][j-1]];
      auxlrc=MeanCells[Cell[i][j-1]];
    }

  //position pin flip in cortex
  pospinflip=-1;
  i=disttoepi+sizeepi+2;
  j=m-1;
  while(!(Tmt[i][j]==2 && (Pin2PrePat[i][j]==1||PinPrePat[i][j]==1)))//if split between pin and pin2 is explicit, cortex is pin2, otherwise it is pin.
    j--;
  pospinflip=j;

  //sizes of vasc cells in OZ
  //2, 4, 8 or 16 cells nearest yLRC1
  int vascsizescellsup[8];
  int vascsizescellsdown[8];
  int counterup;
  int counterdown;
  int qq;
  for(i=0;i<8;i++)
    {
      vascsizescellsup[i]=-1;
      vascsizescellsdown[i]=-1;
    }
  i=disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+1;
  j=yLRC1;
  if(Cell[i][j]==-1)
    j++;
  //printf("just checking: cell %i type %i (should be 9) pos %i,%i\n",Cell[i][j],Tct[i][j],i,j);

  //go up
  counterup=0;
  while(counterup<8 && Tc[i][j]!=3)
    {
      qq=Cell[i][j];
      while(Cell[i][j]==qq)
	j++;
      vascsizescellsup[counterup]=CellHeight[qq];
      counterup++;
      j++;
    }
  //restart, now go down
  j=yLRC1;
  if(Cell[i][j]==-1)
    j+1;
  //we had this cell so first find next one
  qq=Cell[i][j];
  while(Cell[i][j]==qq)
    j--;
  j--;//skip wall
  counterdown=0;
  while(counterdown<8 && Tc[i][j]!=3)
    {
      qq=Cell[i][j];
      while(Cell[i][j]==qq)
	j--;
      vascsizescellsdown[counterdown]=CellHeight[qq];
      counterdown++;
      j--;
    }
  //for(i=0;i<8;i++)
  //printf("cell i %i up %i down %i\n",i,vascsizescellsup[i],vascsizescellsdown[i]);

  double vascmaxnearest=0;
  double vascavgsize2=0;
  double vascavgsize4=0;
  double vascavgsize8=0;
  double vascavgsize16=0;
  double vascavgsize1up=0;
  double vascavgsize2up=0;
  double vascavgsize4up=0;
  double vascavgsize8up=0;
  double vascavgsize1down=0;
  double vascavgsize2down=0;
  double vascavgsize4down=0;
  double vascavgsize8down=0;
  for(i=0;i<8;i++)
    {
      vascavgsize16+=(vascsizescellsup[i]+vascsizescellsdown[i]);
      vascavgsize8up+=vascsizescellsup[i];
      vascavgsize8down+=vascsizescellsdown[i];
      if(i<4)
	{
	  vascavgsize8+=(vascsizescellsup[i]+vascsizescellsdown[i]);
	  vascavgsize4up+=vascsizescellsup[i];
	  vascavgsize4down+=vascsizescellsdown[i];
	}
      if(i<2)
	{
	  vascavgsize4+=(vascsizescellsup[i]+vascsizescellsdown[i]);
	  vascavgsize2up+=vascsizescellsup[i];
	  vascavgsize2down+=vascsizescellsdown[i];
	}
      if(i<1)
	{
	  vascavgsize2+=(vascsizescellsup[i]+vascsizescellsdown[i]);
	  vascavgsize1up+=vascsizescellsup[i];
	  vascavgsize1down+=vascsizescellsdown[i];
	}
      
    }
  if(vascsizescellsup[0]>vascsizescellsdown[0])
    vascmaxnearest=vascsizescellsup[0];
  else
    vascmaxnearest=vascsizescellsdown[0];
  vascavgsize2=(double)vascavgsize2/2.0;
  vascavgsize4=(double)vascavgsize4/4.0;
  vascavgsize8=(double)vascavgsize8/8.0;
  vascavgsize16=(double)vascavgsize16/16.0;
  //vascavgsize1up=(double)vascavgsize1up/1.0;
  vascavgsize2up=(double)vascavgsize2up/2.0;
  vascavgsize4up=(double)vascavgsize4up/4.0;
  vascavgsize8up=(double)vascavgsize8up/8.0;
  //vascavgsize1down=(double)vascavgsize1down/1.0;
  vascavgsize2down=(double)vascavgsize2down/2.0;
  vascavgsize4down=(double)vascavgsize4down/4.0;
  vascavgsize8down=(double)vascavgsize8down/8.0;

  //sizes of peri cells in OZ
  //2, 4, 8 or 16 cells nearest yLRC1
  int perisizescellsup[8];
  int perisizescellsdown[8];
  //int counterup;
  //int counterdown;
  //int qq;
  for(i=0;i<8;i++)
    {
      perisizescellsup[i]=-1;
      perisizescellsdown[i]=-1;
    }
  i=disttoepi+totsizeepi+totsizecort+totsizeendo+1;
  j=yLRC1;
  if(Cell[i][j]==-1)
    j++;
  //printf("just checking: cell %i type %i (should be 8)pos %i,%i\n",Cell[i][j],Tct[i][j],i,j);

  //go up
  counterup=0;
  while(counterup<8 && Tc[i][j]!=3)
    {
      qq=Cell[i][j];
      while(Cell[i][j]==qq)
	j++;
      perisizescellsup[counterup]=CellHeight[qq];
      counterup++;
      j++;
    }
  //restart, now go down
  j=yLRC1;
  if(Cell[i][j]==-1)
    j+1;
  //we had this cell so first find next one
  qq=Cell[i][j];
  while(Cell[i][j]==qq)
    j--;
  j--;//skip wall
  counterdown=0;
  while(counterdown<8 && Tc[i][j]!=3)
    {
      qq=Cell[i][j];
      while(Cell[i][j]==qq)
	j--;
      perisizescellsdown[counterdown]=CellHeight[qq];
      counterdown++;
      j--;
    }

  double perimaxnearest=0;
  double periavgsize2=0;
  double periavgsize4=0;
  double periavgsize8=0;
  double periavgsize16=0;
  double periavgsize1up=0;
  double periavgsize2up=0;
  double periavgsize4up=0;
  double periavgsize8up=0;
  double periavgsize1down=0;
  double periavgsize2down=0;
  double periavgsize4down=0;
  double periavgsize8down=0;
  for(i=0;i<8;i++)
    {
      periavgsize16+=(perisizescellsup[i]+perisizescellsdown[i]);
      periavgsize8up+=perisizescellsup[i];
      periavgsize8down+=perisizescellsdown[i];
      if(i<4)
	{
	  periavgsize8+=(perisizescellsup[i]+perisizescellsdown[i]);
	  periavgsize4up+=perisizescellsup[i];
	  periavgsize4down+=perisizescellsdown[i];
	}
      if(i<2)
	{
	  periavgsize4+=(perisizescellsup[i]+perisizescellsdown[i]);
	  periavgsize2up+=perisizescellsup[i];
	  periavgsize2down+=perisizescellsdown[i];
	}
      if(i<1)
	{
	  periavgsize2+=(perisizescellsup[i]+perisizescellsdown[i]);
	  periavgsize1up+=perisizescellsup[i];
	  periavgsize1down+=perisizescellsdown[i];
	}
    }
  if(perisizescellsup[0]>perisizescellsdown[0])
    perimaxnearest=perisizescellsup[0];
  else
    perimaxnearest=perisizescellsdown[0];
  periavgsize2=(double)periavgsize2/2.0;
  periavgsize4=(double)periavgsize4/4.0;
  periavgsize8=(double)periavgsize8/8.0;
  periavgsize16=(double)periavgsize16/16.0;
  //periavgsize1up=(double)periavgsize1up/1.0;
  periavgsize2up=(double)periavgsize2up/2.0;
  periavgsize4up=(double)periavgsize4up/4.0;
  periavgsize8up=(double)periavgsize8up/8.0;
  //periavgsize1down=(double)periavgsize1down/1.0;
  periavgsize2down=(double)periavgsize2down/2.0;
  periavgsize4down=(double)periavgsize4down/4.0;
  periavgsize8down=(double)periavgsize8down/8.0;

  sprintf(s,"%s/ExtraMeasurements",dirname);
  f=fopen(s,"a");
  fprintf(f,"%i\t%i\t%i\t%f\t%i\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n",t,toplrc,sizetoplrc,auxlrc,pospinflip,perimaxnearest,periavgsize2,periavgsize4,periavgsize8,periavgsize16,periavgsize1up,periavgsize2up,periavgsize4up,periavgsize8up,periavgsize1down,periavgsize2down,periavgsize4down,periavgsize8down,vascmaxnearest,vascavgsize2,vascavgsize4,vascavgsize8,vascavgsize16,vascavgsize1up,vascavgsize2up,vascavgsize4up,vascavgsize8up,vascavgsize1down,vascavgsize2down,vascavgsize4down,vascavgsize8down);
  fclose(f);
  //2-3-4-5 toplrc,sizelrctopcell,auxinthere,pospinflip
  //6-10 peri max,2,4,8,16
  //11-14 peri up
  //15-18 peri down
  //19-23 vasc max,2,4,8,16
  //24-27 vasc up
  //28-31 vasc down
  sprintf(s,"%s/ExtraMeasurements2",dirname);
  f=fopen(s,"a");
  i=disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+1;
  j=yLRC1;
  if(Cell[i][j]!=-1)
    {
      fprintf(f,"%i\t%i\t%i\t%i\n",t,Cell[i][j],CellDivNr[Cell[i][j]],CellTimeLastDiv[Cell[i][j]]);
    }
  fclose(f);
}


void SaveTotAuxin(int t)
{
  FILE *f;
  int i,j;
  char s[900];

  double Auxtot=0;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Auxtot+=Caux1[i][j];

  sprintf(s,"%s/Totalauxin",dirname);
  f=fopen(s,"a");
  fprintf(f,"%i\t%f\t%f\n",t,Auxtot,Auxtot/(n*m));
  fclose(f);
}

void SaveTotPlts(int t)
{
  FILE *f;
  char s[900];

  double Pltstot=0;
  for(int q = 0;q<actcells;q++){
    Pltstot+=Plts[q];
  }
  sprintf(s,"%s/Totalplts",dirname);
  f=fopen(s,"a");
  fprintf(f,"%i\t%f\t%f\n",t,Pltstot,Pltstot/actcells);
  fclose(f);
  
}




void SaveTemp(int t,int II,int JJ)
{
  FILE *f;					
  char s[1000];
  int q;

  q=Cell[II][JJ];
  if (q == -1){q = Cell[II][JJ+1];}
  if (q == -1){q = Cell[II+1][JJ+1];}
  if (q == -1){q = Cell[II-1][JJ];}
  if (q == -1){q = Cell[II-1][JJ-1];}
  if(q!=-1)
    {
      sprintf(s,"%s/TemporalDynamicsPoint%i_%i",dirname,II,JJ);
      f=fopen(s,"a");
      fprintf(f,"%i\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n",t,MeanCells[q],ARF[q],AA[q],BB[q],CC[q],DD[q],min(ARF[q],DD[q]),Plts[q],Pltsexpr[q],Diff[q],ARR1[q],ARR12[q],Yuc3[q]);
      fclose(f);
    }
  else 
  {
	cout << "cell wall has more than 1 height? Or reached beyond some threshold"<<std::endl;
	//exit(1);  
  }
}

void SaveTempQC(int t)
{
  FILE *f;					
  char s[1000];
  int i = n/2-3;
  int j = 0;
  while (Tct[i][j]!=12){
	j++;
  }
  int q = Cell[i][j];
  sprintf(s,"%s/TemporalDynamicsQC%i_%i",dirname,i,j);
  f=fopen(s,"a");
  fprintf(f,"%i\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n",t,MeanCells[q],ARF[q],AA[q],BB[q],CC[q],DD[q],min(ARF[q],DD[q]),Plts[q],Pltsexpr[q],Diff[q],ARR1[q],ARR12[q],Yuc3[q]);
  fclose(f);
}



void SaveLeftRightDiff(int t)
{
  int i,j,q;
  int lefti,righti;
  int leftjaux,rightjaux;
  int leftjaux2,rightjaux2;
  int leftjplt,rightjplt;
  int leftjdiff,rightjdiff;
  FILE *g;
  char s[1000];

  lefti=x0LRC1;
  righti=n-1-lefti;

  i=lefti;
  for(j=0;j<m;j++)
    {
      if(Cell[i][j]!=-1)
	{
	  q=Cell[i][j];
	  if(MeanCells[q]<5)
	    {
	      leftjaux=j;
	      j=2*m;
	    }
	}
    }
  i=righti;
  for(j=0;j<m;j++)
    {
      if(Cell[i][j]!=-1)
	{
	  q=Cell[i][j];
	  if(MeanCells[q]<5)
	    {
	      rightjaux=j;
	      j=2*m;
	    }
	}
    }
  i=lefti;
  for(j=0;j<m;j++)
    {
      if(Cell[i][j]!=-1)
	{
	  q=Cell[i][j];
	  if(MeanCells[q]<10)
	    {
	      leftjaux2=j;
	      j=2*m;
	    }
	}
    }
  i=righti;
  for(j=0;j<m;j++)
    {
      if(Cell[i][j]!=-1)
	{
	  q=Cell[i][j];
	  if(MeanCells[q]<10)
	    {
	      rightjaux2=j;
	      j=2*m;
	    }
	}
    }
  i=lefti;
  for(j=0;j<m;j++)
    {
      if(Cell[i][j]!=-1)
	{
	  q=Cell[i][j];
	  if(Plts[q]<15)
	    {
	      leftjplt=j;
	      j=2*m;
	    }
	}
    }
  i=righti;
  for(j=0;j<m;j++)
    {
      if(Cell[i][j]!=-1)
	{
	  q=Cell[i][j];
	  if(Plts[q]<15)
	    {
	      rightjplt=j;
	      j=2*m;
	    }
	}
    }
  i=lefti;
  for(j=0;j<m;j++)
    {
      if(Cell[i][j]!=-1)
	{
	  q=Cell[i][j];
	  if(Diff[q]>85)
	    {
	      leftjdiff=j;
	      j=2*m;
	    }
	}
    }
  i=righti;
  for(j=0;j<m;j++)
    {
      if(Cell[i][j]!=-1)
	{
	  q=Cell[i][j];
	  if(Diff[q]>85)
	    {
	      rightjdiff=j;
	      j=2*m;
	    }
	}
    }


  sprintf(s,"%s/LeftRightDifferences",dirname);
  g=fopen(s,"a");
  fprintf(g,"%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\n",t,leftjaux,rightjaux,(leftjaux-rightjaux),leftjaux2,rightjaux2,(leftjaux2-rightjaux2),leftjplt,rightjplt,(leftjplt-rightjplt),leftjdiff,rightjdiff,(leftjdiff-rightjdiff));
  fclose(g);
}

void SaveGradCellularRes(int t)
{
  FILE *g;
  int i,ii,j,q;
  char s[1000];

  for(int k=0;k<8;k++)
    {
      if(k==0)
	{
	  //left epidermis
	  i=disttoepi+0.5*totsizeepi;
	  sprintf(s,"%s/Spatialdynamics_epiL_time%i",dirname,t);
	}
      else if(k==1)
	{
	  //right epidermis
	  ii=disttoepi+0.5*totsizeepi;
	  i=n-1-ii;
	  sprintf(s,"%s/Spatialdynamics_epiR_time%i",dirname,t);
	}
      else if(k==2)
	{
	  //left cortex
	  i=disttoepi+totsizeepi+0.5*totsizecort;
	  sprintf(s,"%s/Spatialdynamics_cortexL_time%i",dirname,t);
	}
      else if(k==3)
	{
	  //right cort
	  ii=disttoepi+totsizeepi+0.5*totsizecort;
	  i=n-1-ii;
	  sprintf(s,"%s/Spatialdynamics_cortexR_time%i",dirname,t);
	}
      else if(k==4)
	{
	  //left endodermis
	  i=disttoepi+totsizeepi+totsizecort+0.5*totsizeendo;
	  sprintf(s,"%s/Spatialdynamics_endoL_time%i",dirname,t);
	}
      else if(k==5)
	{
	  //right endodermis
	  ii=disttoepi+totsizeepi+totsizecort+0.5*totsizeendo;
	  i=n-1-ii;
	  sprintf(s,"%s/Spatialdynamics_endoR_time%i",dirname,t);
	}
      else if(k==6)
	{
	  //left vasculature
	  i=disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+0.3*totsizevasc;
	  sprintf(s,"%s/Spatialdynamics_vascL_time%i",dirname,t);
	}
      else if(k==7)
	{
	  //right vasculature
	  ii=disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+0.3*totsizevasc;
	  i=n-1-i;
	  sprintf(s,"%s/Spatialdynamics_vascR_time%i",dirname,t);
	}
      
      
      g=fopen(s,"wb");
      for(j=0;j<m;j++)
	{
	  q=Cell[i][j];
	  if(q>=0)
	    fprintf(g,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%i\n",j,MeanCells[q],ARF[q],AA[q],BB[q],CC[q],DD[q],Plts[q],Pltsexpr[q],Diff[q],CellHeight[q]);
	  else if(Tc[i][j]==3)
	    fprintf(g,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%i\n",j,-10.0,-10.0,-10.0,-10.0,-10,-10.0,-10.0,-10.0,-10.0,-10);
	}
      fclose(g);
      
    }
}



void SaveGradGridRes(int time)
{
  FILE *ff1;
  FILE *ff2;
  char s1[900];
  char s2[900];
  int i,ii,j,k;
  int List[8]={1,5,6,7,8,100,100,100};
  
  for(k=0;k<8;k++)
    {	
      if(k==0)
	{
	  sprintf(s1,"%s/GridResolutionL_RC1_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_RC1_time%i",dirname,time);
	}
      if(k==1)
	{
	  sprintf(s1,"%s/GridResolutionL_Epi_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Epi_time%i",dirname,time);
	  
	}
      if(k==2)
	{
	  sprintf(s1,"%s/GridResolutionL_Cort_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Cort_time%i",dirname,time);
	  
	}
      if(k==3)
	{
	  sprintf(s1,"%s/GridResolutionL_Endo_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Endo_time%i",dirname,time);
	  
	}
      if(k==4)
	{
	  sprintf(s1,"%s/GridResolutionL_Peri_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Peri_time%i",dirname,time);
	  
	}
      if(k==5)
	{
	  sprintf(s1,"%s/GridResolutionL_Vasc1_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Vasc1_time%i",dirname,time);
	  
	}
      if(k==6)
	{
	  sprintf(s1,"%s/GridResolutionL_Vasc2_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Vasc2_time%i",dirname,time);
	  
	}
      if(k==7)
	{
	  sprintf(s1,"%s/GridResolutionL_Vasc3_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Vasc3_time%i",dirname,time);
	  
	}
      
      
      
      
      ff1=fopen(s1,"w");
      ff2=fopen(s2,"w");	
      
      
      
      for(j=0;j< yLRCstraight;j++)
	{
	  
	  i=0;
	  
	  while(Tct[i][j]!=List[k]  && i<xm)
	    i++;
	  
	  if(Tct[i][j]==List[k])
	    {
	      i++;
	      ii=n-(1+i);
	      
	      if(Cell[i][j]!=-1)
		{
		  int current =Cell[i][j];
		  int iplus = i;
		  for (iplus;Cell[iplus][j]==current;iplus++){
		  }
		  int iiminus = n-(1+iplus);
		  fprintf(ff1,"%i\t%i\t%i\t%f\t%f\t%f\t%f\t%f\t%f\n",i,iplus,j,Caux1[i][j],Cpin1M[i][j], Aux1M[i][j],Plts[Cell[i][j]],ARR1[Cell[i][j]],ARR12[Cell[i][j]]);
		  fprintf(ff2,"%i\t%i\t%i\t%f\t%f\t%f\t%f\t%f\t%f\n",iiminus,ii,j,Caux1[ii][j],Cpin1M[ii][j], Aux1M[ii][j],Plts[Cell[ii][j]],ARR1[Cell[ii][j]],ARR12[Cell[ii][j]]);
		}
	    }
	}
      for(j= yzero;j<yEND;j++)
	{
	  if(k==0)
	    i=1;
	  else if(k==1)
	    i=disttoepi+1;
	  else if(k==2)
	    i=totsizeepi+disttoepi+1;
	  else if(k==3)
	    i=totsizecort+totsizeepi+disttoepi+1;
	  else if(k==4)
	    i=totsizeendo+totsizecort+totsizeepi+disttoepi+1;
	  else if(k==5)
	    i=totsizeperi+totsizeendo+totsizecort+totsizeepi+disttoepi+1;
	  else if(k==6)
	    i=sizevasc+totsizeperi+totsizeendo+totsizecort+totsizeepi+disttoepi+1;
	  else if(k==7)
	    i=xm-1;
	  
	  ii=n-(1+i);
	  if(Cell[i][j]!=-1)
	    {
        int current =Cell[i][j];
		  int iplus = i;
		  for (iplus;Cell[iplus][j]==current;iplus++){
		  }
        int iiminus = n-(1+iplus);



	      fprintf(ff1,"%i\t%i\t%i\t%f\t%f\t%f\n",i,iplus,j,Caux1[i][j],Cpin1M[i][j], Aux1M[i][j]);
	      fprintf(ff2,"%i\t%i\t%i\t%f\t%f\t%f\n",iiminus,ii,j,Caux1[ii][j],Cpin1M[ii][j], Aux1M[ii][j]);
	    }
	}
      
      fclose(ff1);
      fclose(ff2);   
	  		
    }		             
 }   

void TrackAuxSpaceTime(int t){

  FILE *f1, *f2,*f3,*f4,*f5,*f6;
  int x, q;
  char s1[1000];
  char s2[1000];
  char s3[1000];
  char s4[1000];
  char s5[1000];
  char s6[1000];
  sprintf(s1,"%s/TrackAuxCorinSpaceTime",dirname);
  sprintf(s2,"%s/TrackAuxendoinSpaceTime",dirname);
  sprintf(s3,"%s/TrackAuxperiinSpaceTime",dirname);
  sprintf(s4,"%s/TrackAuxsidevascinSpaceTime",dirname);
  sprintf(s5,"%s/TrackAuxmidvascinSpaceTime",dirname);
  sprintf(s6,"%s/TrackAuxinSpaceTime",dirname);

  f1=fopen(s1, "a");
  f2=fopen(s2, "a");
  f3=fopen(s3, "a");
  f4=fopen(s4, "a");
  f5=fopen(s5, "a");
  f6=fopen(s6, "a");
  int j = yQC;
  int currentcells[6]  = {-2,-2,-2,-2,-2,-2};
  int celltypes[6] = {6,7,8,9,10,5};
  while (Tc[disttoepi+1][j]!=3 && j < m){
    for( int cellcheck = 0;cellcheck<6;cellcheck++){
      int i=disttoepi;
      while (Tct[i-1][j]!=celltypes[cellcheck] &&i<n/2){
        i++;
        if(Tct[i][j]==celltypes[cellcheck]){// && Tc[i][j]!=2)
          q=Cell[i][j];
          if (q > currentcells[cellcheck]){
            if (Tct[i][j]==6) fprintf(f1, "%i\t%i\t%f\n", t,j,MeanCells[q]);
            if (Tct[i][j]==7) fprintf(f2, "%i\t%i\t%f\n", t,j,MeanCells[q]);
            if (Tct[i][j]==8) fprintf(f3, "%i\t%i\t%f\n", t,j,MeanCells[q]);
            if (Tct[i][j]==9) fprintf(f4, "%i\t%i\t%f\n", t,j,MeanCells[q]);
            if (Tct[i][j]==10){
               //special case scenario that sidevasc and midvasc are the same for just one cell at the bottom
               if (j+1 < yLRCstraight){ //just this once we also add the value to sidevasc
                 fprintf(f4,"%i\t%i\t%f\n", t,j,MeanCells[q]);
               }
               fprintf(f5,"%i\t%i\t%f\n", t,j,MeanCells[q]);}
            if (Tct[i][j]==5) fprintf(f6, "%i\t%i\t%f\n", t,j,MeanCells[q]);

            currentcells[cellcheck] = q;
            break;
		    }
        }   

	   }
    }
    j++;
  }
  fprintf(f1,"\n");
  fprintf(f2,"\n");
  fprintf(f3,"\n");
  fprintf(f4,"\n");
  fprintf(f5,"\n");
  fprintf(f6,"\n");
  fclose(f1);
  fclose(f2);
  fclose(f3);
  fclose(f4);
  fclose(f5);
  fclose(f6);
}




void TrackARRsSpaceTime(int t){
  cout << "startarrs"<<std::endl;
  FILE *f1, *f2,*f3,*f4,*f5,*f6;
  int x, q;
  char s1[1000];
  char s2[1000];
  char s3[1000];
  char s4[1000];
  char s5[1000];
  char s6[1000];
  sprintf(s1,"%s/TrackARR1sexpinSpaceTime",dirname);
  sprintf(s2,"%s/TrackARR12sexpinSpaceTime",dirname);
  sprintf(s3,"%s/TrackARR1seffinSpaceTime",dirname);
  sprintf(s4,"%s/TrackARR1vascexpinSpaceTime",dirname);
  sprintf(s5,"%s/TrackARR12vascexpinSpaceTime",dirname);
  sprintf(s6,"%s/TrackARR1vasceffinSpaceTime",dirname);

  f1=fopen(s1, "a");
  f2=fopen(s2, "a");
  f3=fopen(s3, "a");
  f4=fopen(s4, "a");
  f5=fopen(s5, "a");
  f6=fopen(s6, "a");

  int j = yQC;
  int currentcell = -2;
  while (Tc[disttoepi+1][j]!=3 && j < m){
    int i=disttoepi;
    while (Tct[i-1][j]!=5 &&i<n/2){
      i++;
      if(Tct[i][j]==5){// && Tc[i][j]!=2)
        q=Cell[i][j];
        if (q > currentcell){
          fprintf(f1, "%i\t%i\t%f\n", t,j,ARR1[q]);
          fprintf(f2, "%i\t%i\t%f\n", t,j,ARR12[q]); 
          fprintf(f3, "%i\t%i\t%f\n", t,j,ARR1RGA[q]); 
          currentcell = q;
          break;
        }
      }
	 }
    j++;
  }



  j = yQC;
  currentcell = -2;
  int initialcell = -2;
  while(Tc[disttoepi+1][j]!=3 && j < m){
    int i = disttoepi+1;
    int epicell = Cell[i][j];
    if (initialcell!= -1 && epicell > initialcell){  
      initialcell = epicell;
      double vascarr1=0;
      double vascarr12=0;
      double vascarr1eff=0;
      int vasccells =0;
      while (i<n/2){
        i++;
        q=Cell[i][j];
        if (q!=-1 && q>currentcell){
          if (Tct[i][j]==9){
            vascarr1+=ARR1[q];
            vascarr12+=ARR12[q];
            vascarr1eff+=ARR1RGA[q];
            vasccells++;

          }
          currentcell = q;
        }
      }

      fprintf(f4, "%i\t%i\t%f\n", t,j,vascarr1/vasccells);
      fprintf(f5, "%i\t%i\t%f\n", t,j,vascarr12/vasccells);
      fprintf(f6, "%i\t%i\t%f\n", t,j,vascarr1eff/vasccells);

    }
    j++;
  }


  fprintf(f1,"\n");
  fprintf(f2,"\n");
  fprintf(f3,"\n");
  fprintf(f4,"\n");
  fprintf(f5,"\n");
  fprintf(f6,"\n");

  fclose(f1);
  fclose(f2);
  fclose(f3);
  fclose(f4);
  fclose(f5);
  fclose(f6);
  cout << "finisharrs"<<std::endl;
}

void TrackPltsSpaceTime(int t){
  cout << "starttrackplts"<<std::endl;
  FILE *f1, *f2,*f3,*f4,*f5,*f6,*f7,*f8;
  int x, q;
  char s1[1000];
  char s2[1000];
  char s3[1000];
  char s4[1000];
  char s5[1000];
  char s6[1000];
  char s7[1000];
  char s8[1000];
  sprintf(s1,"%s/TrackPltsinSpaceTime",dirname);
  sprintf(s2,"%s/TrackPltsexpinSpaceTime",dirname);
  sprintf(s3,"%s/TrackPltscorinSpaceTime",dirname);
  sprintf(s4,"%s/TrackPltscorexpinSpaceTime",dirname);
  sprintf(s5,"%s/TrackPltsvascinSpaceTime",dirname);
  sprintf(s6,"%s/TrackPltsvascexpinSpaceTime",dirname);
  sprintf(s7,"%s/TrackPltsaverageinSpaceTime",dirname);
  sprintf(s8,"%s/TrackPltsaverageexpinSpaceTime",dirname);

  f1=fopen(s1, "a");
  f2=fopen(s2, "a");
  f3=fopen(s3, "a");
  f4=fopen(s4, "a");
  f5=fopen(s5, "a");
  f6=fopen(s6, "a");
  f7=fopen(s7, "a");
  f8=fopen(s8, "a");

  int j = yQC;
  int currentcell = -2;
  while (Tc[disttoepi+1][j]!=3 && j < m){
    int i=disttoepi;
    while (Tct[i-1][j]!=5 &&i<n/2){
      i++;
      if(Tct[i][j]==5){// && Tc[i][j]!=2)
        q=Cell[i][j];
        if (q > currentcell){
          fprintf(f1, "%i\t%i\t%f\n", t,j,Plts[q]);
          fprintf(f2, "%i\t%i\t%f\n", t,j,Pltsexpr[q]); 
          currentcell = q;
          break;
        }
      }
	 }
    j++;
  }

  j = yQC;
  currentcell = -2;
  while (Tc[disttoepi+1][j]!=3 && j < m){
    int i=disttoepi+totsizeepi;
    while (Tct[i-1][j]!=6 &&i<n/2){
      i++;
      if(Tct[i][j]==6){// && Tc[i][j]!=2)
        q=Cell[i][j];
        if (q > currentcell){
          fprintf(f3, "%i\t%i\t%f\n", t,j,Plts[q]);
          fprintf(f4, "%i\t%i\t%f\n", t,j,Pltsexpr[q]); 
          currentcell = q;
          break;
        }
      }
	 }
    j++;
  }


  j = yQC;
  currentcell = -2;
  int initialcell = -2;
  while(Tc[disttoepi+1][j]!=3 && j < m){
    int i = disttoepi+1;
    int epicell = Cell[i][j];
    if (initialcell!= -1 && epicell > initialcell){  
      initialcell = epicell;
      double vascplts=0;
      double vascexplts=0;
      double averageplts=0;
      double averageexplts=0;
      int vasccells =0;
      int averagecells = 0;
      while (i<n/2){
        i++;
        q=Cell[i][j];
        if (q!=-1 && q>currentcell){
          averageplts+=Plts[q];
          averageexplts+=Pltsexpr[q];
          averagecells++;
          if (Tct[i][j]==9){
            vascplts+=Plts[q];
            vascexplts+=Pltsexpr[q]; 
            vasccells++;

          }
          currentcell = q;
        }
      }

      fprintf(f5, "%i\t%i\t%f\n", t,j,vascplts/vasccells);
      fprintf(f6, "%i\t%i\t%f\n", t,j,vascexplts/vasccells);
      fprintf(f7, "%i\t%i\t%f\n", t,j,averageplts/averagecells);
      fprintf(f8, "%i\t%i\t%f\n", t,j,averageexplts/averagecells);
    }
    j++;
  }

  fprintf(f1,"\n");
  fprintf(f2,"\n");
  fprintf(f3,"\n");
  fprintf(f4,"\n");
  fprintf(f5,"\n");
  fprintf(f6,"\n");
  fprintf(f7,"\n");
  fprintf(f8,"\n");


  fclose(f1);
  fclose(f2);
  fclose(f3);
  fclose(f4);
  fclose(f5);
  fclose(f6);
  fclose(f7);
  fclose(f8);


  cout << "finishplts"<<std::endl;
}

void TrackCellTypeSpaceTime(int t)
{
   TrackARRsSpaceTime(t); //calculate ARRS

    FILE *f1, *f2,*f3,*f4,*f5;
    int x, q;
    char s1[1000];
    char s2[1000];
    char s3[1000];
    char s4[1000];
    char s5[1000];
    

    //int temp[m];
    double aa;
    double temp;
    double temp_time;
    int temp_size;
    int neighboursize;
    double ratio;
 
    sprintf(s1,"%s/TrackCellTypeinSpaceTime",dirname);

    sprintf(s2,"%s/TrackdivTFinSpaceTime",dirname);
    sprintf(s3,"%s/TrackelongTFinSpaceTime",dirname);
    sprintf(s4,"%s/TrackdivTFvascinSpaceTime",dirname);
    sprintf(s5,"%s/TrackelongTFvascinSpaceTime",dirname);
    f1=fopen(s1, "a");
    f2=fopen(s2, "a");
    f3=fopen(s3, "a");
    f4=fopen(s4, "a");
    f5=fopen(s5, "a");


    int j = yQC;
    int currentcellE = -2;
    int currentcellV = -2;
    while (Tc[disttoepi+1][j]!=3 && j < m){
      int i=disttoepi;
      while (Tct[i-1][j]!=5 &&i<n/2){
        i++;
        if(Tct[i][j]==5){// && Tc[i][j]!=2)
           q=Cell[i][j];
          if (q > currentcellE){
              fprintf(f1, "%i\t%i\t%i\n", t,j,CellZonationState[q]);
              fprintf(f2, "%i\t%i\t%f\n", t,j,divTF[q]);
              fprintf(f3, "%i\t%i\t%f\n", t,j,elongTF[q]);
              currentcellE = q;
          	  break;
		    }
        }
      }
      i=disttolrc+sizetoplrc+totsizeepi+totsizecort+totsizeendo+totsizeperi;

      while (Tct[i-1][j]!=9 &&i<n/2){
        i++;
        if(Tct[i][j]==9){// && Tc[i][j]!=2)
           q=Cell[i][j];
          if (q > currentcellV){
              fprintf(f4, "%i\t%i\t%f\n", t,j,divTF[q]);
              fprintf(f5, "%i\t%i\t%f\n", t,j,elongTF[q]);
              currentcellV = q;
          	  break;
		    }
        }


	   }

	   
      j++;
	}
  fprintf(f1,"\n");
  fprintf(f2,"\n");
  fprintf(f3,"\n");
  fprintf(f4,"\n");
  fprintf(f5,"\n");
  fclose(f1);
  fclose(f2);
  fclose(f3);
  fclose(f4);
  fclose(f5);

}
void Tracktotalmeasures(int t){
  FILE *f1, *f2;
  char s1[1000];
  char s2[1000];


  sprintf(s1,"%s/Tracktotalrootsize",dirname);
  sprintf(s2,"%s/Tracktotalrootnumber",dirname);
    f1=fopen(s1, "a");
    f2=fopen(s2, "a");
    fprintf(f1, "%i\t%i\t%i\n", t,leftsize,rightsize);
    fprintf(f2, "%i\t%i\t%i\n", t,totalrootcellnrL,totalrootcellnrR);   
    
  fclose(f1);
  fclose(f2);

}


