/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "Arrays.hh"
#include "Auxin_params.hh"
#include "Basic_params.hh"
#include "Functions.hh"
#include "Param_config.hh"
#include "Plant_layout.hh"
#include <math.h>





void UpdateMembraneProteinsAndFluxes(int time)
{
  int i,j,k,q,ii;

  static int counter=0;
  FILE *f;
  double invspstep=1.0/(double)(xstep);

  for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
      {
	ii=n-1-i;
	PinPrePat[i][j]=0;
	PinPrePat[ii][j]=0;
	Pin2PrePat[i][j]=0;
	Pin2PrePat[ii][j]=0;
	Aux1PrePat[i][j]=0;
	Aux1PrePat[ii][j]=0;
      }

  #ifndef RESTART
  if(time==0){
    for(q=0;q<actcells;q++){
      if(CellMaxJ[q]<yLRCstraight){
	     CellZonationState[q]=0;
      }
	   else if(CellMaxJ[q]<yLRCstraight+nrcellsstraightMZ*(HMZ+1)){
	     CellZonationState[q]=30;
      }
      else if(CellMaxJ[q]<yLRCstraight+nrcellsstraightMZ*(HMZ+1)+nrcellsNDIV*(Hndivi+1)  ){
	     CellZonationState[q]=50;
      }
      else if(CellMaxJ[q]<yLRCstraight+nrcellsstraightMZ*(HMZ+1)+nrcellsNDIV*(Hndivi+1)+nrcellsEZ*(HEZ/2+1)){
	     CellZonationState[q]=60;
      }
      else if (CellMaxJ[q]<yLRCstraight+nrcellsstraightMZ*(HMZ+1)+nrcellsNDIV*(Hndivi+1)+nrcellsEZ*(HEZ/2+1)+nrcellsDZ*(HDZ+1)){
        CellZonationState[q]=100;
        Diff[q]=100;
      }
      else if (CellMaxJ[q]<yLRCstraight+nrcellsstraightMZ*(HMZ+1)+nrcellsNDIV*(Hndivi+1)+nrcellsEZ*(HEZ/2+1)+nrcellsDZ*(HDZ+1)+nrcellsHJ*(HJZ+1)){
	     CellZonationState[q]=200; //Hypocotyljunctionstate!!
        Diff[q] = 100;
      }
	 }
  }
  #endif

#ifdef POLAR
  int kk,ll;
  int mindist;
  double disttresh;
  double distance; 
  for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
      {
      ii=n-1-i;
      q=Cell[i][j]; 
      if ( (Tc[i][j]==2) &&  ((j<yLRCstraight)||
         ( !((Tc[i][j+1]==2 && Tc[i+1][j]==2) || (Tc[i][j+1]==2 && Tc[i-1][j]==2)|| (Tc[i][j-1]==2 && Tc[i+1][j]==2) || (Tc[i][j-1]==2 && Tc[i-1][j]==2)) ) )
         ){//membrane     
        if(Tct[i][j]<=4){	 //lateral root cap
		    if(Tct[i][j]==1 &&Tc[i][CellMaxJ[q]+3]==3 ){//if(Tct[i][j]==1 && j>yLRCstraight)//if(Tct[i][j]==1 &&CellMinJ[q]>yLRCstraight+5*HMZ+5) //CHANGED KIRSTEN short wedge
            if(Tmt[i][j]==1){//up  
              Pin2PrePat[i][j]=1;
              Pin2PrePat[ii][j]=1;
              }
            else if(Tmt[i][j]==3){//inward membrane
              disttresh=(int)(0.35*(CellMaxJ[Cell[i][j]]-CellMinJ[Cell[i][j]]));
              distance =10000;
              for(kk=max(0,i-100/xstep);kk<=min(n-1,i+100/xstep);kk++){ //min max to not go outside the field
                for(ll=CellMinJ[Cell[i][j]];ll<=CellMaxJ[Cell[i][j]];ll++){//cells always inside the field
                  if(Tmt[kk][ll]==1  && Cell[kk][ll]==Cell[i][j]){//upward membrane of same cell
                    distance=abs(kk-i)+abs(ll-j);
                    if(distance<=disttresh){
                      kk=10000;
                      ll=10000;//to stop searching if distance below threshold is found
                      }
                    }
                  }
                }
              if(distance<=disttresh)
                {
                Pin2PrePat[i][j]=1;
                Pin2PrePat[ii][j]=1;
                }
              else
                {
                  Pin2PrePat[i][j]=0.0005;
                  Pin2PrePat[ii][j]=0.0005;
                 }
              }
            else if (Tmt[i][j]==4){
              if (CellZonationState[q] < 60){
                Pin2PrePat[i][j]=0.0;
                Pin2PrePat[ii][j]=0.0;
                }
              else{
                Pin2PrePat[i][j]=0.0;
                Pin2PrePat[ii][j]=0.0;   
                }
              }  
            }
          else if (Tct[i][j]==1){ //other lateral root cap cells
            if (j<yLRCstraight){
              if(Tmt[i][j]==1){//up
			       Pin2PrePat[i][j]=0.5;
			       Pin2PrePat[ii][j]=0.5;
		        }
              else if (Tmt[i][j]==3){
             	 Pin2PrePat[i][j]=0.35;
			       Pin2PrePat[ii][j]=0.35;
              }
            }
		    else { //Tct == 1 and y > yLRCstraight
		      if(Tmt[i][j]==1){//up
			     Pin2PrePat[i][j]=1;
			     Pin2PrePat[ii][j]=1;
		        }
              else if(Tmt[i][j]==3 || Tmt[i][j]==4)//in and out
		        {
			      //reducing this to 0.05 leads to substantial
			      //auxin increase, increasing to 0.5 leads to
			      //auxin decrease
                 
              //Pin2PrePat[i][j]=0.15;
              //Pin2PrePat[ii][j]=0.15;
                     
              if(Tmt[i][j]==3)
                {
                if (Tc[i][CellMaxJ[q]+3]==3){
                Pin2PrePat[i][j]=0.3;
                 Pin2PrePat[ii][j]=0.3;      
                }
                else if (CellZonationState[q] >50){
                  Pin2PrePat[i][j]=0.3;
                  Pin2PrePat[ii][j]=0.3;
                  }
                else{
                  Pin2PrePat[i][j]=0.0005;
                  Pin2PrePat[ii][j]=0.0005;
                  } 
                }
                else{ //tmt == 4
                  if (CellZonationState[q] > 50){
                    Pin2PrePat[i][j]=0.0;
                    Pin2PrePat[ii][j]=0.0;
                  }
                  else{
                    Pin2PrePat[i][j]=0.01;
                    Pin2PrePat[ii][j]=0.01;
                  }

                  }
                } 
		        }
            } 
            else{ // LRC cells that are not the outer layer
			  if(Tmt[i][j]==1){//up
			     Pin2PrePat[i][j]=0.5;
			     Pin2PrePat[ii][j]=0.5;
		        }
		      else if (Tmt[i][j]==3){//inside
				 Pin2PrePat[i][j]=0.35;
                 Pin2PrePat[ii][j]=0.35;
			  }
           } 
          }


        else if(Tct[i][j]==5){//epidermis
          if(Tmt[i][j]==1){//up
		    //strange,
		    //changing second if into else if and hence changing
		    //PIN2 upward in lowermost part from 0.5 to 0.1 
		    //results in even more auxin acc
		    //maintaining the else if and now changing the
		    //0.1 to 0.5 and 0.5 to 0.75 hardly helps
		    //note that because initially we did not have the
		    //else if, in lower part it was 0.5 anyways so
		    //the only real change is in the slightly less
		    //lower part, from 0.5 to 0.75 which has little
		    //effect
            if(CellZonationState[q]<60){//CellMinJ[q]<yLRCstraight+5*HMZ+5)
              Pin2PrePat[i][j]=1;
              Pin2PrePat[ii][j]=1;
              }
           else if(CellZonationState[q]<100){//CellMinJ[q]<yLRCstraight+5*HMZ+5)
              Pin2PrePat[i][j]=1;
              Pin2PrePat[ii][j]=1;
              }
            else if(CellZonationState[q]==100){ //DZ  
	           Pin2PrePat[i][j]=0.4;
              Pin2PrePat[ii][j]=0.4;
		        }
		      else if(CellZonationState[q]==200){ //HJZ
              Pin2PrePat[i][j]=1;
              Pin2PrePat[ii][j]=1;
		        }
            }




          else if(Tmt[i][j]==2){//down
            if(CellZonationState[q]==0){
              if(j<=(int)(arad1*i+brad1)){
			       Pin2PrePat[i][j]=0.25;
			       Pin2PrePat[ii][j]=0.25;
                } 
              else if(CellMinJ[q]<=yLRCstraight){
                Pin2PrePat[i][j]=0.25;
                Pin2PrePat[ii][j]=0.25;
                } 
              }
            }

          else if(Tmt[i][j]==3 || Tmt[i][j]==4)
		      {
		      if(CellMinJ[q]<=yLRCstraight){
              if(Tmt[i][j]==3){
                Pin2PrePat[i][j]=0.1;
			       Pin2PrePat[ii][j]=0.1;
			       }
			     else{ //out
			       Pin2PrePat[i][j]=0.1;
			       Pin2PrePat[ii][j]=0.1;
			       }
		        } 


		      else{ //cellminj>ylrcstraight
              if(Tmt[i][j]==3){
                int celli = Cell[i-1][j];
                int cellii = Cell[ii+1][j];
               if ( CellMinJ[Cell[i][CellMinJ[q]-3]]<=yLRCstraight+3 ){

                  Pin2PrePat[i][j]=0.1; 
                  Pin2PrePat[ii][j]=0.1;
                }
               else if ( CellZonationState[q]<=50 ){

                  Pin2PrePat[i][j]=0.1; 
                  Pin2PrePat[ii][j]=0.1;
                }
                else if (CellZonationState[q]<100||CellZonationState[q]==200) {
                  Pin2PrePat[i][j]=0.1;
                  Pin2PrePat[ii][j]=0.1;
                  }
			       else if (CellZonationState[q]==100){
                  Pin2PrePat[i][j]=0.1; 
                  Pin2PrePat[ii][j]=0.1;
                  }
                }
              else if(Tmt[i][j]==4){
                if ( CellZonationState[q]<50  ){

                  Pin2PrePat[i][j]=0.1;
                  Pin2PrePat[ii][j]=0.1;
                  }

                else if (CellZonationState[q]==50){
                  Pin2PrePat[i][j]=0.10;
                  Pin2PrePat[ii][j]=0.10;
                  }
                else{ 
                  Pin2PrePat[i][j]=0.0;
                  Pin2PrePat[ii][j]=0.0;
                  }
			       }
		        }


		      }
	       }
        else if(Tct[i][j]==6){//cortex 
        //normal
		    if(Tmt[i][j]==1){//up
             if( CellMaxJ[q]<yLRCstraight && CellMaxJ[q]+3>yLRCstraight){
              Pin2PrePat[i][j]=.1;
              Pin2PrePat[ii][j]=.1;
             }
           #ifndef ARRPINFLIP
             else if (CellZonationState[q]>50){ 
           #else
             else if (ARRpassed[q]||CellZonationState[q]==200){
           #endif
             if(CellZonationState[q]<100){//until past DZ   
              Pin2PrePat[i][j]=1;
			     Pin2PrePat[ii][j]=1;
		        }
		      else if(CellZonationState[q]==100){ //DZ
              Pin2PrePat[i][j]=.4;
              Pin2PrePat[ii][j]=.4;
              }
            else if(CellZonationState[q]==200){ //HJ
              Pin2PrePat[i][j]=1;
              Pin2PrePat[ii][j]=1;
		        }
            }
          }
          if(Tmt[i][j]==2){
           #ifndef ARRPINFLIP
             if (CellZonationState[q]<=50){ 
           #else
             if (ARRpassed[q]==false && CellZonationState[q]<200){
           #endif
              Pin2PrePat[i][j]=1;
              Pin2PrePat[ii][j]=1;
              }
            }
          else if(Tmt[i][j]==3 ){//|| Tmt[i][j]==4){
            if(CellZonationState[q]==0){
		        if(j<=(int)(arad1*i+brad1)){
                Pin2PrePat[i][j]=0.5;
                Pin2PrePat[ii][j]=0.5;
                }
              } 

             else if (CellZonationState[q]<=50){ 

              Pin2PrePat[i][j]=0.1;
              Pin2PrePat[ii][j]=0.1;
              }
            else if (CellZonationState[q]==50){ 

              Pin2PrePat[i][j]=0.1;
              Pin2PrePat[ii][j]=0.1;
              }
            else{
              Pin2PrePat[i][j]=0.1;
              Pin2PrePat[ii][j]=0.1;
              }
            }
          else if(Tmt[i][j]==4 ){
            if(CellZonationState[q]==0){
		        if(j<=(int)(arad1*i+brad1)){
                Pin2PrePat[i][j]=0.5;
                Pin2PrePat[ii][j]=0.5;
                }
              }          
             else if (CellZonationState[q]==100){
              Pin2PrePat[i][j]=0.1;
              Pin2PrePat[ii][j]=0.1;
              }
            else if (CellZonationState[q]<60){
              #ifdef PINOUTWARDCORTEX
                Pin2PrePat[i][j]=0.1;
                Pin2PrePat[ii][j]=0.1;
              #else 
                Pin2PrePat[i][j]=0.0;
                Pin2PrePat[ii][j]=0.0;
              #endif
              }
            else{ //if cellzonationstate is 60 or 200
              Pin2PrePat[i][j]=0.0;
              Pin2PrePat[ii][j]=0.0;
              }           
             }
          }
        else if(Tct[i][j]==7){//endodermis different
          if(Tmt[i][j]==2){//down
             if (CellZonationState[q]==100){ 
              
              PinPrePat[i][j]=0.35;
              PinPrePat[ii][j]=0.35;                     
              }
            else {
              PinPrePat[i][j]=1;
              PinPrePat[ii][j]=1;
              }
            }
          else if(Tmt[i][j]==1 ){ //Up

             if (CellZonationState[q]==100){ 
                PinPrePat[i][j]=0.1;
                PinPrePat[ii][j]=0.1;                   
              }
              else if ( j<=(int)(arad1*i+brad1)){
                PinPrePat[i][j]=0.5;//1;
                PinPrePat[ii][j]=0.5;//1;
              }
              else{
                PinPrePat[i][j]=0.0;
                PinPrePat[ii][j]=0.0;   
              }
            } 
          else if(Tmt[i][j]==3 || Tmt[i][j]==4){//in and out
              if(CellZonationState[q]==0){ 
                if(j<=(int)(arad1*i+brad1)){
                  PinPrePat[i][j]=0.7;
                  PinPrePat[ii][j]=0.7;
                  } 
                else{  
                  if(Tmt[i][j]==3){//in
                    PinPrePat[i][j]=0.35;
                    PinPrePat[ii][j]=0.35;
                    }
                  else{//out
                    PinPrePat[i][j]=0.15;
                    PinPrePat[ii][j]=0.15;
                    }
                  }
                } 
              else if (CellZonationState[q]==100){   
               if(Tmt[i][j]==3){//in
                 PinPrePat[i][j]=0.1;
                 PinPrePat[ii][j]=0.1;
                 }
                else{//out
                 PinPrePat[i][j]=0.1;
                 PinPrePat[ii][j]=0.1;
                 }



               }
               else { //if not differentiation zone
                 if(Tmt[i][j]==3){//in
                   PinPrePat[i][j]=0.35;
                   PinPrePat[ii][j]=0.35;
                 }
                 else{ //out
                   PinPrePat[i][j]=0.0;
                   PinPrePat[ii][j]=0.0;
                 }
               } 
             }
		     }
	       
        else if(Tct[i][j]==8){//pericycle
          if(Tmt[i][j]==1 ){ //Up
            PinPrePat[i][j]=0;
            PinPrePat[ii][j]=0;
          }
          else if(Tmt[i][j]==2){//down
           if (   CellMinJ[   Cell[i][CellMinJ[q]-3]     ] >= yLRCstraight+(HMZ*4-1)     ){
              PinPrePat[i][j]=1;
              PinPrePat[ii][j]=1;
		        }

            #ifndef PIN1QCAPOLAR
		      else if (CellMinJ[q]> yQC ){
		        PinPrePat[i][j]=1;
		        PinPrePat[ii][j]=1;
              }
            #endif
            #ifdef PIN1QCAPOLAR
            else if (CellMinJ[q] >= yLRCstraight+(HMZ*4-1)){
              PinPrePat[i][j]=0.75;
              PinPrePat[ii][j]=0.75;
		        }
	         else if (CellMinJ[q]> yQC ){
		        PinPrePat[i][j]=0.5;
		        PinPrePat[ii][j]=0.5;
		        }		    
            #endif
            }
          else if(Tmt[i][j]==3 || Tmt[i][j]==4){
            if(CellZonationState[q]==0){//if(CellMinJ[q]<yR+8*HMZ+8)//if(CellZonationState[q]==0)
              if(j<=(int)(arad1*i+brad1)){
                PinPrePat[i][j]=0.7;
                PinPrePat[ii][j]=0.7;
                }
              }
            else{
			     if(Tmt[i][j]==3){//in
                if (CellMinJ[q] < yLRCstraight+(HMZ*4-1)){
                  PinPrePat[i][j]=0.5;
                  PinPrePat[ii][j]=0.5;
                  }
                else {
                  PinPrePat[i][j]=0.1;
                  PinPrePat[ii][j]=0.1;
                  }
			       }
              else {//out
                PinPrePat[i][j]=0.0;
                PinPrePat[ii][j]=0.0;
                }
              }
		      }
 
          }	   

        else if(Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11){//vasculature
          if(Tmt[i][j]==2){//down 
            if (   CellMinJ[   Cell[i][CellMinJ[q]-3]     ] >= yLRCstraight+(HMZ*4-1)     ){
              PinPrePat[i][j]=1;
              PinPrePat[ii][j]=1;
		        }

            #ifndef PIN1QCAPOLAR
		      else if (CellMinJ[q]> yQC ){
		        PinPrePat[i][j]=1;
		        PinPrePat[ii][j]=1;
              }
            #endif
            #ifdef PIN1QCAPOLAR
            else if (CellMinJ[q] >= yLRCstraight+(HMZ*4-1)){
              PinPrePat[i][j]=0.75;
              PinPrePat[ii][j]=0.75;
		        }
	         else if (CellMinJ[q]> yQC ){
		        PinPrePat[i][j]=0.5;
		        PinPrePat[ii][j]=0.5;
		        }		    
            #endif
	         else if (CellMinJ[q] < yQC-5 ){// to make sure you're below the QC
              PinPrePat[i][j]=0.4;
              PinPrePat[ii][j]=0.4;
              }
            }
          else if(Tmt[i][j]==3 || Tmt[i][j]==4){
            if(CellMinJ[q]<yLRCstraight+(HMZ*4-1)){//if(CellMinJ[q]<yR+8*HMZ+8)//if(CellMinJ[q]<yR)//
              if(j<=(int)(arad1*i+brad1)){
                PinPrePat[i][j]=0.4;
                PinPrePat[ii][j]=0.4;
                }
              else{
                if(Tmt[i][j]==3){//in
                  PinPrePat[i][j]=0.4;
                  PinPrePat[ii][j]=0.4;
                  }
                else{//out
                  PinPrePat[i][j]=0.4;
                  PinPrePat[ii][j]=0.4;
                  }
                }
              }
              else if (Tmt[i][j]==3){ //inside 
                if (CellZonationState[q]>=30){
                  PinPrePat[i][j]=0.0;
                  PinPrePat[ii][j]=0.0;
                  }
                }
              else if(Tmt[i][j]==4){
                if (CellZonationState[q]>=30){
                  PinPrePat[i][j]=0.1;
                  PinPrePat[ii][j]=0.1;
                  }
                }
              }
          #ifdef PIN1QCAPOLAR
		    else if (Tmt[i][j]==1){ //up
		      if (CellMinJ[q]> yQC && CellMinJ[q]< yLRCstraight+(HMZ*4-1) ){
		        PinPrePat[i][j]=0.00;
		        PinPrePat[ii][j]=0.00;
		      }
		    }
          #endif
        }


	   else if(Tct[i][j]==12)//QC
	     {
		  if(j==CellMinJ[Cell[i][j]])//works better than Tmt[i][j]==4//downward
		    {
		    PinPrePat[i][j]=0.5;
		    PinPrePat[ii][j]=0.5;
		    }
	     }
	   else if(Tct[i][j]==13)//columella bottom layer
	     {
		  if(Tmt[i][j]<4)//not downward
		    {    
		    PinPrePat[i][j]=0.5;
		    PinPrePat[ii][j]=0.5;
		    }
	    }
	    else if(Tct[i][j]>13)//columella except bottom layer, identical
	      {
		if(Tct[i][j]<17)//not the topmost one
		  {
		    if(Tmt[i][j]<=3)//left,right and up
		      {
			PinPrePat[i][j]=0.75 *.5;
			PinPrePat[ii][j]=0.75*.5;
		      }
		    else 
		      {
			PinPrePat[i][j]=1*.5;
			PinPrePat[ii][j]=1*.5;
		      }
		  }
		else
		  {
		    if(Tmt[i][j]==1)
		      {
			PinPrePat[i][j]=0.75*.5;
			PinPrePat[ii][j]=0.75*.5;
		      }
		    else
		      {
			PinPrePat[i][j]=1*.5;
			PinPrePat[ii][j]=1*.5;
		      }
		  }
	      }	   
	  }
      }
#endif


#ifndef AUX1EPIVAL
  double aux1divzone = 0.1;
#endif

//calculate how many cells there are in the epidermis, and where their borders are.
  i = disttoepi+1;
  j = yLRCstraight+2;
  int currentcell = Cell[i][j];
  int cellnumbers = 1;
  int cellborders[7] ={};
  while (j <  yLRCstraight+(8*HMZ-2) && cellnumbers < 7){
    if ((Cell[i][j]!=currentcell) && (Cell[i][j]!=-1)){
      cellborders[cellnumbers-1]= j;
      cellnumbers +=1;
      currentcell = Cell[i][j];
    }
    j++;
  }

//


 
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)
	  {
	    q=Cell[i][j];
	    ii=n-1-i;
       int midcelli = (CellMinI[q]+CellMaxI[q])/2;
       int midcellj = (CellMinJ[q]+CellMaxJ[q])/2;
	    if(Tct[midcelli][midcellj]==12)//QC
	      Aux1PrePat[i][j]=3;//was 2 and then(red)
	    if(Tct[midcelli][midcellj]>12 && Tct[midcelli][midcellj]!=13)//columella behalve onderste laag
	      Aux1PrePat[i][j]=3;//(yellow)
	    else if(Tct[i][j]==13)
	      Aux1PrePat[i][j]=1;//(purplish)
	    else if (Tct[midcelli][midcellj]==1 && CellMinJ[q] < yLRCstraight ) //lrc 1
	      Aux1PrePat[i][j]=1;
       else if (Tct[midcelli][midcellj]==1 )
         Aux1PrePat[i][j]=1;//1;//1. changed to get less auxin accumulation in LRC
       else if (Tct[midcelli][midcellj]>=2 && Tct[midcelli][midcellj]<=4)//lrc2-4
	      Aux1PrePat[i][j]=1;//1;//1. changed to get less auxin accumulation in LRC
	    else if( (j<yLRCstraight )&& (((i<n/2 && j<=(int)(arad1*i+brad1)) || (i>n/2 && j<=(arad1*ii+brad1))) && (Tct[midcelli][midcellj]<8 || Tct[midcelli][midcellj]>12)) )
	      Aux1PrePat[i][j]=1;
	    else if(Tct[i][j]==5)//epi
	      {
          if( CellMinJ[q] <yLRCstraight+(8*HMZ-2) ){
            if (cellnumbers <=4){
              Aux1PrePat[i][j]=1;
            }
            else if (cellnumbers <=5){
              if( CellMinJ[q]< cellborders[3]){
                Aux1PrePat[i][j]=1;
              }
              else{
                Aux1PrePat[i][j]=0.55;
              }
            }
            else if (cellnumbers <=6){
              if( CellMinJ[q]< cellborders[3]){
                Aux1PrePat[i][j]=1;
              }
              else if( CellMinJ[q]< cellborders[4]){
                Aux1PrePat[i][j]=.7;
              }
              else{
                Aux1PrePat[i][j]=0.4;
              }
            }

            else if (cellnumbers <=7){
              if( CellMinJ[q]< cellborders[3]){
                Aux1PrePat[i][j]=1;
              }
              else if (CellMinJ[q]< cellborders[4]){
                Aux1PrePat[i][j]=0.775;
              }
              else if (CellMinJ[q]< cellborders[5]){
                Aux1PrePat[i][j]=0.55;
              }
              else {
                Aux1PrePat[i][j]=0.225;
              }
            }
            }
		    else if (CellZonationState[Cell[i][j]]<50){
		      Aux1PrePat[i][j]=aux1divzone;
	       }		   
		    else if ( (CellZonationState[Cell[i][j]]<60) && (  CellZonationState[
                                                               Cell[i][(CellMaxJ[    
                                                                 Cell[i][(CellMaxJ[    
                                                                   Cell[i][(CellMaxJ[
                                                                    (Cell[i][( CellMaxJ[q]+3 )])
                                                                   ]+3)]
                                                                 ]+3)]
                                                               ]+3) ]
                                                             ]<60  ) )    { //denotes that you are 50 but four cells above you aren't EZ yet.
		    Aux1PrePat[i][j]=aux1divzone;
	     }
	     else {//if  (CellZonationState[Cell[i][j]]>50){ // denotes that you are either higher than 50 or the last cell before the EZ.
		    Aux1PrePat[i][j]=1;
	     }
	    }
	      
	    else if(Tct[i][j]==6)//cortex
	      {
         
         if(CellMinJ[q] <yLRCstraight-1 ){
		    Aux1PrePat[i][j]=1;
         }
         else if(CellMinJ[q] <yLRCstraight+3 ){
		    Aux1PrePat[i][j]=1;
         }

         else if(CellMinJ[q] <cellborders[1]){
		    Aux1PrePat[i][j]=0.7;
         }
         else if(CellMinJ[q] <cellborders[2]){
		    Aux1PrePat[i][j]=0.4;
         }
         
         else   
         if( CellZonationState[q]<50){
		     Aux1PrePat[i][j]=0.1;
         }
         else if( (CellZonationState[q]<60) &&( CellZonationState[Cell[i][( CellMaxJ[q]+3 )]]<60  ) ){
		     Aux1PrePat[i][j]=0.1;
         }
         else{
		    Aux1PrePat[i][j]=0.5;
         }

	    }
      
	    else if(Tct[i][j]==7)//endodermis
	      {
          if(CellMinJ[q] <yLRCstraight+3 ){
		     Aux1PrePat[i][j]=1;
           }
         else{
	        Aux1PrePat[i][j]=0;
           }
         Aux1PrePat[i][j]=0;
         }
          
	    else if(Tct[i][j]>=8 && Tct[i][j]<=11)//peri and vasc
	      {
       
       if (
            (  
               ( (maxmzheight-yLRCstraight) < 14*1.5*HMZ ) &&
                                                  (  CellMinJ[q]<( ((maxmzheight-yLRCstraight)*1)/2+yLRCstraight )  
                                                   ||(CellMinJ[q]-3)<yLRCstraight
                                                   ||(CellMinJ[Cell[midcelli][(CellMinJ[q]-3)] ]-3  )<yLRCstraight  
                                                  )
            ) 
        

            || ( ( (maxmzheight-yLRCstraight) >= 14*1.5*HMZ )   &&   (  CellMinJ[q]<( (7*1.5*HMZ)+yLRCstraight )  )   )     
 
         ){

         #ifndef HIGHAUX1VASCMZ
	        Aux1PrePat[i][j]=1;
         #else
	        Aux1PrePat[i][j]=vascmzaux1;

         #endif
       }
       
       else if (
       (j > yLRCstraight )&& (
          (  ( (maxmzheight-yLRCstraight) < 14*1.5*HMZ ) && ( (CellMinJ[Cell[midcelli][CellMinJ[q]-3]]) <( ((maxmzheight-yLRCstraight)*1)/2+yLRCstraight) )  )  
          || ( ( (maxmzheight-yLRCstraight) >= 14*1.5*HMZ )   &&   ( ( CellMinJ[Cell[midcelli][CellMinJ[q]-3]]  )  <( (7*1.5*HMZ)+yLRCstraight )  )   )     
           || (CellMinJ[Cell[midcelli][
                              CellMinJ[
                                      Cell[midcelli][
                                                    CellMinJ[q]-3
                                                    ]
                                      ]-3
                             ]
                        ]-3 < yLRCstraight)
         

         )   ) {
       
         #ifndef HIGHAUX1VASCMZ
	        Aux1PrePat[i][j]=1;
         #else
	        Aux1PrePat[i][j]=(vascmzaux1+vascmzaux1+1.)/3.;

         #endif
       } 
       else if (
       (j > yLRCstraight )&& (
          (  ( (maxmzheight-yLRCstraight) < 14*1.5*HMZ ) && ( (CellMinJ[ Cell[midcelli][(CellMinJ[Cell[midcelli][CellMinJ[q]-3]])-3]]) <( ((maxmzheight-yLRCstraight)*1)/2+yLRCstraight) )  )  
          || ( ( (maxmzheight-yLRCstraight) >= 14*1.5*HMZ )   &&   ( ( CellMinJ[Cell[midcelli][(CellMinJ[Cell[midcelli][CellMinJ[q]-3]])-3]  ] )  <( (7*1.5*HMZ)+yLRCstraight )  )   )     
           || (CellMinJ[Cell[midcelli][
                              CellMinJ[
                                      Cell[midcelli][
                                                    CellMinJ[q]-3
                                                    ]
                                      ]-3
                             ]
                        ]-3 < yLRCstraight)
         

         )   ) {
       
         #ifndef HIGHAUX1VASCMZ
	        Aux1PrePat[i][j]=1;
         #else
	        Aux1PrePat[i][j]=(vascmzaux1+1+1.)/3.;

         #endif
       }
       


       
       else{
       
	      Aux1PrePat[i][j]=1;
       } 
       }
	  }
      }
 
  //update membrane protein concentration based on new prepatterns and gene expression levels
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]!=-1)//grid point inside cell
	  {
	    q=Cell[i][j]; 	    
	    Cpin1M[i][j]=Pin[q]*PinPrePat[i][j];
	    Cpin2M[i][j]=Pin2[q]*Pin2PrePat[i][j];
	    Aux1M[i][j]=Aux1[q]*Aux1PrePat[i][j];	  
	  }
	else
	  {
	    Cpin1M[i][j]=0;
	    Cpin2M[i][j]=0;
	    Aux1M[i][j]=0;
	  }   	
      }
  
  
  //update auxin fluxes and metabolism based on new tissue architecture and gene expression levels
  //auxin prod, decay, diff, flux rates:
  //direction 0 i+1;
  //direction 1 i-1;
  //direction 2 j+1;
  //direction 3 j-1;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	daux[i][j]=0;//default no decay
	paux[i][j]=0;//default no production
	for(k=0;k<4;k++)
	  {	  
	    Inf[i][j][k]=0;
	    Eff[i][j][k]=0;
	    DiffAuxC[i][j][k]=0;
	  }
      }
 


  //daux depends on Gh317 gene expression
  for(i=0;i<n;i++){
    for(j=0;j<m;j++)
      {
	q=Cell[i][j];
	if(q!=-1){
     #ifdef HCJBASEAUXDEGR
     if( CellZonationState[q] < 200){
	  //daux[i][j]=dAux*(1+Gh317eff*Gh317[q]/100.+Gh33eff*Gh33[q]/100.); 
       daux[i][j]=dAux+dA_GH317*Gh317[q]+dA_GH33*Gh33[q]; 
     }
     else{
       daux[i][j]=dAux*1.5;
     }
     
     #else
     //daux[i][j]=dAux*(1+Gh317eff*Gh317[q]/100.+Gh33eff*Gh33[q]/100.); 
       daux[i][j]=dAux+dA_GH317*Gh317[q]+dA_GH33*Gh33[q]; 
     #endif
       
      }
    }
  }
  //paux depends on Yuc3 expression
  double volfactor;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
	{
	  ii=n-1-i;
	  q=Cell[i][j];
	  if(q!=-1)
	    { 

	      if(Tct[i][j]==12 || Tct[i][j]==17)//QC and columella layer 5: okay
		volfactor=50;
	      else if(j<yQC+2*HMZ+2 && 
		      (Tct[i][j]==8|| Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11))//this part I changed
		volfactor=50;
	      else if((j<yQC+HMZ+1 && (j<(int)(arad3*i+brad3)  && j<(int)(arad3*ii+brad3))) && 
		      (Tct[i][j]==5 || Tct[i][j]==6 || Tct[i][j]==7 || Tct[i][j]==8))//epi-peri cells next to QC, added j<yQC+HMZ+1 to delimit it to lowest peri cell            	
		volfactor=50;
	      else if(Tct[i][j]>=1 && Tct[i][j]<=4)//Lateral Root Cap
		volfactor=15;
	      else if(Tct[i][j] > 12 && Tct[i][j]< 17 )//Columella layers 1-4 from the bottom
		volfactor = 25;
	      else if (j< yLRCstraight)
           volfactor = 1;
         else
		volfactor=0;
	      //paux[i][j]=(pAux+ volfactor*meristemfactor*pAux+maxyucAuxeff*(Yuc3[q]/100.)*pAux)*HMZ/CellHeight[q];
           paux[i][j]=(pAux+ volfactor*meristemfactor*pAux+  pAyuc*Yuc3 )*HMZ/CellHeight[q];
	    }
	}


#ifdef AUXINAPPLICATION
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(i==0 || i==n-1 || j==0)
	  paux[i][j]+=1.00/xstep;
      }
#endif

#ifndef SATURATION
  //Set Inf from Aux1
  #ifndef ARRAUXINFLUX
    double pasinfreg = 1.0;
  #endif
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)
	  {
	    //nearest cell:
	    if(i!=0){
	      if(Tc[i-1][j]==2){
        	# ifdef ARRAUXINFLUX
	        double pasinfreg = ARRbaseinflux + (ARRmaxinflux-ARRbaseinflux)* positivesigmoid(ARR1[Cell[i-1][j]]+ARR12[Cell[i-1][j]],ARFKM_ARRinflux,200);
         #endif
		     Inf[i][j][1]=(-inf*pasinfreg-aux1fac*Aux1M[i-1][j])*invspstep;
         }
       } 
	    if(i!=n-1){	
	      if(Tc[i+1][j]==2){
        	# ifdef ARRAUXINFLUX
	        double pasinfreg = ARRbaseinflux + (ARRmaxinflux-ARRbaseinflux)* positivesigmoid(ARR1[Cell[i+1][j]]+ARR12[Cell[i+1][j]],ARFKM_ARRinflux,200);
         #endif
	        Inf[i][j][0]=(-inf*pasinfreg-aux1fac*Aux1M[i+1][j])*invspstep;
         }
       }
	    if(j!=0){
	      if(Tc[i][j-1]==2){
        	# ifdef ARRAUXINFLUX
	        double pasinfreg = ARRbaseinflux + (ARRmaxinflux-ARRbaseinflux)* positivesigmoid(ARR1[Cell[i][j-1]]+ARR12[Cell[i][j-1]],ARFKM_ARRinflux,200);
         #endif
		     Inf[i][j][3]=(-inf*pasinfreg-aux1fac*Aux1M[i][j-1])*invspstep;
         }
       }
	    if(j!=m-1){
	      if(Tc[i][j+1]==2){
        	# ifdef ARRAUXINFLUX
	        double pasinfreg = ARRbaseinflux + (ARRmaxinflux-ARRbaseinflux)* positivesigmoid(ARR1[Cell[i][j+1]]+ARR12[Cell[i][j+1]],ARFKM_ARRinflux,200);
         #endif
		     Inf[i][j][2]=(-inf*pasinfreg-aux1fac*Aux1M[i][j+1])*invspstep;
         }
       }
	  }
	else if(Tc[i][j]==2)
	  {
   	# ifdef ARRAUXINFLUX
	     double pasinfreg = ARRbaseinflux + (ARRmaxinflux-ARRbaseinflux)* positivesigmoid(ARR1[Cell[i][j]]+ARR12[Cell[i][j]],ARFKM_ARRinflux,200);
      #endif
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		Inf[i][j][0]=(inf*pasinfreg+aux1fac*Aux1M[i][j])*invspstep;
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		Inf[i][j][1]=(inf*pasinfreg+aux1fac*Aux1M[i][j])*invspstep;
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		Inf[i][j][2]=(inf*pasinfreg+aux1fac*Aux1M[i][j])*invspstep;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		Inf[i][j][3]=(inf*pasinfreg+aux1fac*Aux1M[i][j])*invspstep;
	  }
      }
  //Set Eff from Tc
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//wall, neighboring membrane
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==2)
		Eff[i][j][0]=(eff*invspstep+pPUMP*((pinfac1*Cpin1M[i+1][j]+(1-pinfac1)*100*PinPrePat[i+1][j])+(pinfac2*Cpin2M[i+1][j]+(1-pinfac2)*100*Pin2PrePat[i+1][j]))*invspstep);
	    if(i!=0)
	      if(Tc[i-1][j]==2)
		Eff[i][j][1]=(eff*invspstep+pPUMP*((pinfac1*Cpin1M[i-1][j]+(1-pinfac1)*100*PinPrePat[i-1][j])+(pinfac2*Cpin2M[i-1][j]+(1-pinfac2)*100*Pin2PrePat[i-1][j]))*invspstep);
	    if(j!=m-1)
	      if(Tc[i][j+1]==2)
		Eff[i][j][2]=eff*invspstep+pPUMP*((pinfac1*Cpin1M[i][j+1]+(1-pinfac1)*100*PinPrePat[i][j+1])+(pinfac2*Cpin2M[i][j+1]+(1-pinfac2)*100*Pin2PrePat[i][j+1]))*invspstep;;
	    if(j!=0)
	      if(Tc[i][j-1]==2)
		Eff[i][j][3]=eff*invspstep+pPUMP*((pinfac1*Cpin1M[i][j-1]+(1-pinfac1)*100*PinPrePat[i][j-1])+(pinfac2*Cpin2M[i][j-1]+(1-pinfac2)*100*Pin2PrePat[i][j-1]))*invspstep;;
	  }
	else if(Tc[i][j]==2)//membrane, neighboring wall
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		Eff[i][j][0]=(-eff*invspstep-pPUMP*((pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])+(pinfac2*Cpin2M[i][j]+(1-pinfac2)*100*Pin2PrePat[i][j]))*invspstep);
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		Eff[i][j][1]=(-eff*invspstep-pPUMP*((pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])+(pinfac2*Cpin2M[i][j]+(1-pinfac2)*100*Pin2PrePat[i][j]))*invspstep);
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		Eff[i][j][2]=-eff*invspstep-pPUMP*((pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])+(pinfac2*Cpin2M[i][j]+(1-pinfac2)*100*Pin2PrePat[i][j]))*invspstep;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		Eff[i][j][3]=-eff*invspstep-pPUMP*((pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])+(pinfac2*Cpin2M[i][j]+(1-pinfac2)*100*Pin2PrePat[i][j]))*invspstep;
	  }
      }
#endif




  //Set DiffAuxC from Tc
  double DauxCC=0.5*DauxC*tstep/(xstep*xstep);
  double DauxWW=0.5*DauxW*tstep/(xstep*xstep);
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//wall grid point: only diffuse towards other wall grid point
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		DiffAuxC[i][j][0]=DauxWW;
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		DiffAuxC[i][j][1]=DauxWW;
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		DiffAuxC[i][j][2]=DauxWW;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		DiffAuxC[i][j][3]=DauxWW;
	  }
	else if(Tc[i][j]==1 || Tc[i][j]==2)//cell grid point: only diffuse towards other cell grid point
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==1 || Tc[i+1][j]==2)
		DiffAuxC[i][j][0]=DauxCC;
	    if(i!=0)
	      if(Tc[i-1][j]==1 || Tc[i-1][j]==2)
		DiffAuxC[i][j][1]=DauxCC;
	    if(j!=m-1)
	      if(Tc[i][j+1]==1 || Tc[i][j+1]==2)
		DiffAuxC[i][j][2]=DauxCC;
	    if(j!=0)
	      if(Tc[i][j-1]==1 || Tc[i][j-1]==2)
		DiffAuxC[i][j][3]=DauxCC;
	  }
      }


  //Now we need to adjust for getting a normal influx/efflux relation between root and shoot
#ifdef NORMALSHOOTROOTCONNECTION
  double auxinvirtlayer=auxinflux;
#endif
#ifdef SHOOTROOTCUT
  double auxinvirtlayer=auxinflux*0.5*exp(-0.003*time);
#endif
#ifdef LOWAUXIN_FROMSHOOT 
  double auxinvirtlayer=auxinflux*0.5;
#endif
  int jbot;
  int jstore;

  for(i=0;i<n;i++)
    for(j=m-1;j>=0;j--)
      {
	
	if(Tc[i][j]==0 && Tc[i][j-1]==2)//wall above cell
	  {
	    if(Tct[i][j-1]==5 || Tct[i][j-1]==6 || Tct[i][j-1]==7)//efflux to shoot from epi/cort/endo
	      {		  
		jbot=CellMinJ[Cell[i][j-1]]-1;
		daux[i][j]=0.1;
	
	      }
	    else if(Tct[i][j-1]==8 || Tct[i][j-1]==9||Tct[i][j-1]==10||Tct[i][j-1]==11)//influx from shoot to pericycle and vasculature
	      {
         paux[i][j]=(eff*invspstep+pPUMP*100*1*invspstep)*auxinvirtlayer;

	      }
	    break;
	  }
      }
}
