/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "Arrays.hh"
#include "Auxin_params.hh"
#include "Basic_params.hh"
#include "Functions.hh"
#include "Param_config.hh"
#include "Plant_layout.hh"


void InitPlantFromFile(int startt)
{
  FILE *f;
  int i,j,q;
  char fname2[800];
  float temp;
  int temp2;

  sprintf(fname2,"%s/DumpPlantStateAt%.10d",dirname,startt);
  f=fopen(fname2,"r");

  //tissue architecture and cell numbering
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      fscanf(f,"%i ",&(Tc[i][j]));
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      fscanf(f,"%i ",&(Tct[i][j]));
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      fscanf(f,"%i ",&(Tmt[i][j]));
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      fscanf(f,"%i ",&(Cell[i][j]));

  //location of transporters
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	fscanf(f,"%f ",&temp);
	PinPrePat[i][j]=temp;
      }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	fscanf(f,"%f ",&temp);
	Pin2PrePat[i][j]=temp;
      }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	fscanf(f,"%f ",&temp);
	Aux1PrePat[i][j]=temp;
      }

  //auxin levels
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	fscanf(f,"%f ",&temp);
	Caux1[i][j]=temp;

      }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      MeanCells[q]=temp;
    }

  //gene expression levels
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      ARF[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      AA[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      BB[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      CC[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      DD[q]=temp;
    }

  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      Plt[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      Yuc3[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      ARR1[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      ARR12[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      divTF[q]=temp;
    }
     for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      elongTF[q]=temp;
    }
    
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      Gh317[q]=temp;
    }

  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      Pin[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      Pin2[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      Plts[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      Aux1[q]=temp;
    }



  //miscelleanous
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%i ",&TimePrevGrowth[q]);
    }
  for(q=0;q<totcells;q++)
    {
      fscanf(f,"%f ",&temp);
      Diff[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
	  fscanf(f,"%i ",&CellZonationState[q]);

    }
  for(q=0;q<totcells;q++)
    {
	  fscanf(f,"%i ",&CellsInOZ[q]);
    }

  for(q=0;q<totcells;q++)
    {
	  fscanf(f,"%f ",&temp); 
      Gh33[q]=temp;
    }
  for(q=0;q<totcells;q++)
    {
	  fscanf(f,"%i ",&CellDivNr[q]);
    }
  //nr of cells
  fscanf(f,"%i",&actcells);
  fclose(f);

  FindCellCorners();

  FindCellSizes();
  UpdateMembraneProteinsAndFluxes(0);


}

void WritePlantToFile(int time)
{
  FILE *f;
  int i,j,q;
  char fname[800];
  sprintf(fname,"%s/DumpPlantStateAt%.10d",dirname,time);
  f=fopen(fname,"w");

  
  //tissue architecture and cell numbering
  for(i=0;i<n;i++)
    {
      for(j=0;j<m;j++)
	fprintf(f,"%i\t",Tc[i][j]);
      fprintf(f,"\n");
    }
  for(i=0;i<n;i++)
    {
      for(j=0;j<m;j++)
	fprintf(f,"%i\t",Tct[i][j]);
      fprintf(f,"\n");
    }
  for(i=0;i<n;i++)
    {
      for(j=0;j<m;j++)
	fprintf(f,"%i\t",Tmt[i][j]);
      fprintf(f,"\n");
    }
  for(i=0;i<n;i++)
    {
      for(j=0;j<m;j++)
	fprintf(f,"%i\t",Cell[i][j]);
      fprintf(f,"\n");
    }
  
  //location of transporters
  for(i=0;i<n;i++)
    {
      for(j=0;j<m;j++)
	fprintf(f,"%f\t",PinPrePat[i][j]);
      fprintf(f,"\n");
    }
  for(i=0;i<n;i++)
    {
      for(j=0;j<m;j++)
	fprintf(f,"%f\t",Pin2PrePat[i][j]);
      fprintf(f,"\n");
    }
  for(i=0;i<n;i++)
    {
      for(j=0;j<m;j++)
	fprintf(f,"%f\t",Aux1PrePat[i][j]);
      fprintf(f,"\n");
    }
   
  //auxin levels
  for(i=0;i<n;i++)
    {
    for(j=0;j<m;j++){
	  fprintf(f,"%f\t",Caux1[i][j]);
    }
      fprintf(f,"\n");
    }
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",MeanCells[q]);
  fprintf(f,"\n");
  
  //gene expression levels
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",ARF[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",AA[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",BB[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",CC[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",DD[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",Plt[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",Yuc3[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",ARR1[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",ARR12[q]);
  fprintf(f,"\n");
 
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",divTF[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",elongTF[q]);
  fprintf(f,"\n");
    
  
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",Gh317[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",Pin[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",Pin2[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",Plts[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",Aux1[q]);
  fprintf(f,"\n");
  
  
  //miscelleanous
  for(q=0;q<totcells;q++)
    fprintf(f,"%i\t",TimePrevGrowth[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",Diff[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%i\t",CellZonationState[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%i\t",CellsInOZ[q]);
  fprintf(f,"\n");
  
  for(q=0;q<totcells;q++)
    fprintf(f,"%f\t",Gh33[q]);
  fprintf(f,"\n");
  for(q=0;q<totcells;q++)
    fprintf(f,"%i\t",CellDivNr[q]);
  fprintf(f,"\n");
  //nr of cells
  fprintf(f,"%i\n",actcells);
  fclose(f);

}
